//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include <QtSql>
#include <QMessageBox>
#include <iostream>
#include <QComboBox>
#include "utils.h"




#include "DBTools.h"
//-----------------------------------------------------------------------------
DBHandler::DBHandler()
{
    db=QSqlDatabase::addDatabase("QMYSQL");
    db.setHostName("localhost");
    db.setDatabaseName("finanval");
    db.setUserName("finanval");
    db.setPassword("passfinanval");
}

//-----------------------------------------------------------------------------
bool DBHandler::conectar()
{
    if (!db.open())
    {
        QMessageBox::critical(0,"Database Error",db.lastError().text());
        return false;
    }
    return true;
}

//-----------------------------------------------------------------------------
void DBHandler::commit()
{
    this->db.commit();
}

//-----------------------------------------------------------------------------
void DBHandler::rollback()
{
    this->db.rollback();
}

//-----------------------------------------------------------------------------
int DBHandler::getDealsCount()
{
    QSqlQuery query;
    query.exec("SELECT count(*) from deals");
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return 0;
    }
    query.next();
    return query.value(0).toInt();
}

//-----------------------------------------------------------------------------
QSqlQuery *DBHandler::getColumnsFromTable(QString tableName)
{
    QSqlQuery *query=NULL;
    QString StringQuery("SELECT COLUMN_NAME FROM information_schema.COLUMNS WHERE TABLE_SCHEMA LIKE \"finanval\" AND TABLE_NAME =  \"");

    StringQuery.append(tableName);
    StringQuery.append("\"");
    query=new QSqlQuery(StringQuery);

    return query;
}

//-----------------------------------------------------------------------------
QStringList *DBHandler::getColumnsListFromTable(QString tableName)
{
    QSqlQuery *query=NULL;
    QStringList *result = new QStringList;
    query=this->getColumnsFromTable(tableName);
    while (query->next())
    {
        result->append(query->value(0).toString());
    }
    delete query;
    return result;
}

//-----------------------------------------------------------------------------
QSqlQuery *DBHandler::getDataFromTable(QString tableName)
{
    QSqlQuery *query=NULL;
    QString queryTXT("SELECT * FROM ");

    queryTXT.append(tableName);
    query=new QSqlQuery(queryTXT);

    return query;
}

//-----------------------------------------------------------------------------
bool DBHandler::getDataFromTable(QComboBox *combo, QString tableName)
{
    QSqlQuery *query=NULL;
    QString queryTXT("SELECT * FROM ");

    if (!combo)
        combo=new QComboBox();

    queryTXT.append(tableName);
    query=new QSqlQuery(queryTXT);
    while (query->next())
    {
        combo->addItem(query->value(0).toString());
    }
    delete query;
    return true;
}

//-----------------------------------------------------------------------------
QString *DBHandler::getFamilyFromProduct(QString product)
{
    QSqlQuery query;
    QString *ret=new QString;
    query.prepare("SELECT ID_FAMILIA FROM familia_tipo_producto where ID_PRODUCTO=:product");
    query.bindValue(":product",product);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        *ret="";
        return ret;
    }
    while (query.next())
    {
        *ret=query.value(0).toString();
    }

    return ret;
}

//-----------------------------------------------------------------------------
bool DBHandler::getTiposFamilia(QComboBox *combo, QString familia)
{
    QSqlQuery *query= new QSqlQuery;
    query->prepare("SELECT ID_PRODUCTO FROM familia_tipo_producto where ID_FAMILIA=:familia");
    query->bindValue(":familia",familia);
    if (!combo)
        combo=new QComboBox();
    int numItems=-1;
    numItems=combo->count();
    for (int i=numItems-1; i>=0; i--)
        combo->removeItem(i);

    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query->lastError().text());
        return false;
    }
    while (query->next())
    {
        combo->addItem(query->value(0).toString());
    }
    delete query;
    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::updateCartera(QTreeWidget *tree)
{
    if (tree)
        tree->clear();

    tree->setHeaderLabel("MI CARTERA");

    QList<QTreeWidgetItem *> listaItems;
    QSqlQuery *queryFamilias=getFamilies();
    if (!queryFamilias)
        return false;
    while (queryFamilias->next())
    {
        QString familia(queryFamilias->value(0).toString());
        QTreeWidgetItem *familiaItem = new QTreeWidgetItem(tree);
        familiaItem->setText(0, familia);
        listaItems.append(familiaItem);
        QSqlQuery *queryProdFamilia=getFamilyProducts(familia);
        while (queryProdFamilia->next())
        {
            QString producto(queryProdFamilia->value(0).toString());
            QTreeWidgetItem *prodFamiliaItem = new QTreeWidgetItem(familiaItem);
            prodFamiliaItem->setText(0, producto);
            QSqlQuery *queryDeals=getDeals(producto);
            while (queryDeals->next())
            {
                QString deal(queryDeals->value(0).toString());
                QTreeWidgetItem *dealItem = new QTreeWidgetItem(prodFamiliaItem,TIPO_PRODUCTO);
                dealItem->setText(0, deal);
            }
            delete queryDeals;
        }
        delete queryProdFamilia;
    }
    delete queryFamilias;
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::getDataFromTable(QTableWidget *tabla, QString tableName)
{
    int numColumns=0;
    QSqlQuery *queryColumns=NULL;
    QSqlQuery *query=NULL;
    QStringList *tableTitle;

    if (!tabla)
        tabla = new QTableWidget();

    tableTitle=this->getColumnsListFromTable(tableName);
    numColumns=tableTitle->count();
    tabla->setColumnCount(numColumns);

    tabla->setHorizontalHeaderLabels(*tableTitle);
    delete tableTitle;

    query=this->getDataFromTable(tableName);
    tabla->setRowCount(query->numRowsAffected());
    int actualRow=0;
    while (query->next())
    {
        for (int actualColumn=0; actualColumn<numColumns; actualColumn++)
        {
            tabla->setItem(actualRow, actualColumn, new QTableWidgetItem(query->value(actualColumn).toString()));
            Qt::ItemFlags eFlags = tabla->item(actualRow,actualColumn)->flags();
            eFlags &= ~Qt::ItemIsEditable;
            tabla->item(actualRow,actualColumn)->setFlags(eFlags);
        }
        actualRow++;
    }
    tabla->resizeColumnsToContents();

    if (queryColumns)
        delete queryColumns;

    if (query)
        delete query;
    if (tabla->rowCount())
        tabla->selectRow(0);
    return true;

}

//-----------------------------------------------------------------------------
QSqlQuery *DBHandler::getDataFromDepositos()
{
    QSqlQuery *query=new QSqlQuery("SELECT productos.ID, productos.TIPO, productos.CURRENCY_ISO, depositos.PLAZO, depositos.PERIODO, productos.QUOTE FROM productos, depositos WHERE productos.id = depositos.id");
    return query;
}

//-----------------------------------------------------------------------------
QSqlQuery *DBHandler::getDataFromFuturos()
{
    QSqlQuery *query=new QSqlQuery("SELECT productos.ID, productos.TIPO, productos.CURRENCY_ISO, futuros.MONTHS, futuros.IMM_DATE, productos.QUOTE FROM productos, futuros WHERE productos.id = futuros.id");
    return query;
}

//-----------------------------------------------------------------------------
QSqlQuery *DBHandler::getDataFromFras()
{
    QSqlQuery *query=new QSqlQuery("SELECT productos.ID, productos.TIPO, productos.CURRENCY_ISO, fras.MONTHSTOSTART, fras.MONTHSTOEND, productos.QUOTE FROM productos, fras WHERE productos.id = fras.id");
    return query;
}

//-----------------------------------------------------------------------------
QSqlQuery *DBHandler::getDataFromSwaps()
{
    QSqlQuery *query=new QSqlQuery("SELECT productos.ID, productos.TIPO, productos.CURRENCY_ISO, swaps.PLAZO, swaps.PERIODO, swaps.FIXED_BUS_CONVENTION, swaps.FIXED_CAL_CONVENTION, swaps.FIXEDLEG_FRECUENCY, swaps.FLOATING_INDEX, productos.QUOTE FROM productos, swaps WHERE productos.id = swaps.id");
    return query;
}

//-----------------------------------------------------------------------------
bool DBHandler::getDataFromTiposCambio(QTableWidget *tabla)
{
    int numColumns=0;
    QSqlQuery query;
    QString tituloTabla;
    int rowsAffected=0;


    if (!tabla)
        tabla = new QTableWidget();

    numColumns=3;
    tituloTabla.append("DIVISA DOMESTICA; DIVISA FORANEA; TIPO");
    tabla->setColumnCount(numColumns);
    DoubleSpinBoxDelegate *delegate = new DoubleSpinBoxDelegate;
    tabla->setItemDelegateForColumn(numColumns-1, delegate);
    tabla->setHorizontalHeaderLabels(tituloTabla.split(";"));
    QString txt;

    query.exec("SELECT * FROM tipos_cambio");
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    rowsAffected=query.size();
    tabla->setRowCount(rowsAffected);
    int actualRow=0;
    while (query.next())
    {
        for (int actualColumn=0; actualColumn<numColumns; actualColumn++)
        {
            txt=query.value(actualColumn).toString();
            QTableWidgetItem *item = new QTableWidgetItem(txt);
            if (actualColumn!=numColumns-1)
            {

                Qt::ItemFlags eFlags = item->flags();
                eFlags &= ~Qt::ItemIsEditable;
                item->setFlags(eFlags);
            }
            else
            {
                item->setTextColor("Blue");
            }
            tabla->setItem(actualRow, actualColumn, item );
        }
        actualRow++;
    }
    tabla->resizeColumnsToContents();
    if (tabla->rowCount())
        tabla->selectRow(0);
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::getDataFromProductType(QTableWidget *tabla,QString tipo)
{
    int numColumns=0;
    QSqlQuery *queryColumns=NULL;
    QSqlQuery *query=NULL;
    QString tituloTabla;
    int rowsAffected=0;


    if (!tabla)
        tabla = new QTableWidget();

    if (tipo=="Deposito")
    {
        numColumns=6;
        tituloTabla.append("IDENTIFICADOR;TIPO;DIVISA;PLAZO;PERIODO;MARKET QUOTE");
        query=this->getDataFromDepositos();
    }
    else if (tipo=="Futuro")
    {
        numColumns=6;
        tituloTabla.append("IDENTIFICADOR;TIPO;DIVISA;MESES;FECHA IMM;MARKET QUOTE");
        query=this->getDataFromFuturos();
    }
    else if (tipo=="Fra")
    {
        numColumns=6;
        tituloTabla.append("IDENTIFICADOR;TIPO;DIVISA;MESES INICIO;MESES FIN;MARKET QUOTE");
        query=this->getDataFromFras();
    }
    else if (tipo=="Swap")
    {
        numColumns=10;
        tituloTabla.append("IDENTIFICADOR;TIPO;DIVISA;PLAZO;PERIODO;CONV.FESTIVOS;CONV.CALENDARIO;FRECUENCIA FIJA;INDICE;MARKET QUOTE");
        query=this->getDataFromSwaps();
    }
    else
        return false;

    tabla->setColumnCount(numColumns);
    //AL PRINCIPIO ERA DINAMICO PERO CAUSABA UN LEAK
    //DoubleSpinBoxDelegate *delegate = new DoubleSpinBoxDelegate;
    DoubleSpinBoxDelegate delegate;
    tabla->setItemDelegateForColumn(numColumns-1, &delegate);
    tabla->setHorizontalHeaderLabels(tituloTabla.split(";"));
    QString txt;

    rowsAffected=query->size();
    tabla->setRowCount(rowsAffected);
    int actualRow=0;
    while (query->next())
    {
        for (int actualColumn=0; actualColumn<numColumns; actualColumn++)
        {
            txt=query->value(actualColumn).toString();
            QTableWidgetItem *item = new QTableWidgetItem(txt);
            if (actualColumn!=numColumns-1)
            {

                Qt::ItemFlags eFlags = item->flags();
                eFlags &= ~Qt::ItemIsEditable;
                item->setFlags(eFlags);
            }
            else
            {
                item->setTextColor("Blue");
            }
            tabla->setItem(actualRow, actualColumn, item );
        }
        actualRow++;
    }
    tabla->resizeColumnsToContents();

    if (queryColumns)
        delete queryColumns;

    if (query)
        delete query;

    if (tabla->rowCount())
        tabla->selectRow(0);

    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::getDataFromProductType(QComboBox *combo, QString tipo)
{
    QSqlQuery *query=NULL;

    if (!combo)
        combo=new QComboBox();

    int numItems=combo->count();
    for (int i=numItems-1; i>=0; i--)
        combo->removeItem(i);

    QString queryTXT("SELECT ID FROM ");
    if (tipo=="Deposito")
    {
        queryTXT.append("depositos");
    }
    else if (tipo=="Futuro")
    {
        queryTXT.append("futuros");
    }
    else if (tipo=="Fra")
    {
        queryTXT.append("fras");
    }
    else if (tipo=="Swap")
    {
        queryTXT.append("depositos");
    }


    query=new QSqlQuery(queryTXT);
    query->exec();
    while (query->next())
    {
        combo->addItem(query->value(0).toString());
    }
    delete query;
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::getFijacionesFromIndice(QTableWidget *tabla,QString indice)
{
    int numColumns=0;
    QSqlQuery query;
    QString tituloTabla, txt;
    int rowsAffected=0;


    if (!tabla)
        tabla = new QTableWidget();

    numColumns=2;
    tituloTabla.append("FECHA; VALOR");

    query.prepare("SELECT FECHA, VALOR FROM fijaciones WHERE INDICE=:indice");
    query.bindValue(":indice", indice);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }


    tabla->setColumnCount(numColumns);
    tabla->setHorizontalHeaderLabels(tituloTabla.split(";"));


    rowsAffected=query.size();
    tabla->setRowCount(rowsAffected);
    int actualRow=0;
    while (query.next())
    {
        for (int actualColumn=0; actualColumn<numColumns; actualColumn++)
        {
            txt=query.value(actualColumn).toString();
            QTableWidgetItem *item = new QTableWidgetItem(txt);
            Qt::ItemFlags eFlags = item->flags();
            eFlags &= ~Qt::ItemIsEditable;
            item->setFlags(eFlags);
            tabla->setItem(actualRow, actualColumn, item );
        }
        actualRow++;
    }
    tabla->resizeColumnsToContents();

    if (tabla->rowCount())
        tabla->selectRow(0);

    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::getDataFromCurve(QTableWidget *tabla, QString curveID)
{
    QSqlQuery query;
    int index=-1, row=-1;
    QString startDate, tituloCurvas;

    for (int i=tabla->rowCount()-1; i >= 0; --i)
        tabla->removeRow(i);

    tabla->setColumnCount(7);
    tituloCurvas.append("PRODUCT_ID;TIPO;DIVISA;START DATE;PLAZO;PERIODO;MARKET QUOTE");
    tabla->setHorizontalHeaderLabels(tituloCurvas.split(";"));
    QDate today = QDate::currentDate();
    startDate=today.toString(Qt::ISODate);

    //Primero cargamos los depos
    query.prepare("SELECT cp.ID_PRODUCTO, p.TIPO, p.CURRENCY_ISO, d.PLAZO, d.PERIODO, p.QUOTE from curvas c INNER JOIN curva_producto cp ON c.ID = cp.ID_CURVA INNER JOIN productos p ON cp.ID_PRODUCTO = p.ID INNER JOIN depositos d ON cp.ID_PRODUCTO = d.ID where c.ID=:curvaID");
    query.bindValue(":curvaID", curveID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    while (query.next())
    {
        row=tabla->rowCount();
        tabla->insertRow(row);

        index=query.record().indexOf("ID_PRODUCTO");
        tabla->setItem(row, 0, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("TIPO");
        tabla->setItem(row, 1, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("CURRENCY_ISO");
        tabla->setItem(row, 2, new QTableWidgetItem(query.value(index).toString()));
        tabla->setItem(row, 3, new QTableWidgetItem(startDate));
        index=query.record().indexOf("PLAZO");
        tabla->setItem(row, 4, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("PERIODO");
        tabla->setItem(row, 5, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("QUOTE");
        tabla->setItem(row, 6, new QTableWidgetItem(query.value(index).toString()));
        tabla->item(row,6)->setTextColor("Blue");
        tabla->resizeColumnsToContents();
    }

    //Despues cargamos los futuros
    query.prepare("SELECT cp.ID_PRODUCTO, p.TIPO, p.CURRENCY_ISO, f.MONTHS, p.QUOTE from curvas c INNER JOIN curva_producto cp ON c.ID = cp.ID_CURVA INNER JOIN productos p ON cp.ID_PRODUCTO = p.ID INNER JOIN futuros f ON cp.ID_PRODUCTO = f.ID where c.ID=:curvaID");
    query.bindValue(":curvaID", curveID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    while (query.next())
    {
        row=tabla->rowCount();
        tabla->insertRow(row);

        index=query.record().indexOf("ID_PRODUCTO");
        tabla->setItem(row, 0, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("TIPO");
        tabla->setItem(row, 1, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("CURRENCY_ISO");
        tabla->setItem(row, 2, new QTableWidgetItem(query.value(index).toString()));
        tabla->setItem(row, 3, new QTableWidgetItem(startDate));
        index=query.record().indexOf("MONTHS");
        tabla->setItem(row, 4, new QTableWidgetItem(query.value(index).toString()));
        tabla->setItem(row, 5, new QTableWidgetItem("Mes"));
        index=query.record().indexOf("QUOTE");
        tabla->setItem(row, 6, new QTableWidgetItem(query.value(index).toString()));
        tabla->item(row,6)->setTextColor("Blue");
        tabla->resizeColumnsToContents();
    }

    //Seguimos con fras
    query.prepare("SELECT cp.ID_PRODUCTO, p.TIPO, p.CURRENCY_ISO, f.MONTHSTOSTART, f.MONTHSTOEND, p.QUOTE from curvas c INNER JOIN curva_producto cp ON c.ID = cp.ID_CURVA INNER JOIN productos p ON cp.ID_PRODUCTO = p.ID INNER JOIN fras f ON cp.ID_PRODUCTO = f.ID where c.ID=:curvaID");
    query.bindValue(":curvaID", curveID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    while (query.next())
    {
        row=tabla->rowCount();
        tabla->insertRow(row);

        index=query.record().indexOf("ID_PRODUCTO");
        tabla->setItem(row, 0, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("TIPO");
        tabla->setItem(row, 1, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("CURRENCY_ISO");
        tabla->setItem(row, 2, new QTableWidgetItem(query.value(index).toString()));
        tabla->setItem(row, 3, new QTableWidgetItem(startDate));
        index=query.record().indexOf("MONTHSTOSTART");
        int start=query.value(index).toInt();
        index=query.record().indexOf("MONTHSTOEND");
        int end=query.value(index).toInt();
        tabla->setItem(row, 4, new QTableWidgetItem(start+end));
        tabla->setItem(row, 5, new QTableWidgetItem("Mes"));
        index=query.record().indexOf("QUOTE");
        tabla->setItem(row, 6, new QTableWidgetItem(query.value(index).toString()));
        tabla->item(row,6)->setTextColor("Blue");
        tabla->resizeColumnsToContents();
    }

    //Terminamos con swaps
    query.prepare("SELECT cp.ID_PRODUCTO, p.TIPO, p.CURRENCY_ISO, s.PLAZO, s.PERIODO, p.QUOTE from curvas c INNER JOIN curva_producto cp ON c.ID = cp.ID_CURVA INNER JOIN productos p ON cp.ID_PRODUCTO = p.ID INNER JOIN swaps s ON cp.ID_PRODUCTO = s.ID where c.ID=:curvaID");
    query.bindValue(":curvaID", curveID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    while (query.next())
    {
        row=tabla->rowCount();
        tabla->insertRow(row);

        index=query.record().indexOf("ID_PRODUCTO");
        tabla->setItem(row, 0, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("TIPO");
        tabla->setItem(row, 1, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("CURRENCY_ISO");
        tabla->setItem(row, 2, new QTableWidgetItem(query.value(index).toString()));
        tabla->setItem(row, 3, new QTableWidgetItem(startDate));
        index=query.record().indexOf("PLAZO");
        tabla->setItem(row, 4, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("PERIODO");
        tabla->setItem(row, 5, new QTableWidgetItem(query.value(index).toString()));
        index=query.record().indexOf("QUOTE");
        tabla->setItem(row, 6, new QTableWidgetItem(query.value(index).toString()));
        tabla->item(row,6)->setTextColor("Blue");
        tabla->resizeColumnsToContents();
    }

    if (tabla->rowCount())
        tabla->selectRow(0);

    return true;
}

//-----------------------------------------------------------------------------
QString *DBHandler::getCalendarFromCurrency(QString divisa)
{
    QSqlQuery query;
    QString *calendario=new QString();
    query.prepare("SELECT CALENDAR_ID from divisas where CURRENCY_ISO=:divisa");
    query.bindValue(":divisa",divisa);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        *calendario="";
        return calendario;
    }
    query.next();
    *calendario=query.value(0).toString();
    return calendario;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertHolidayIntoCalendario(QString calendar, QDate date)
{
    QSqlQuery query;
    QStringList listaDeleted;
    QStringList listaAdded;
    QString addHolidays, deleteHolidays;
    int index=-1;

    getStringFieldFromTableforID("calendarios", "CALENDAR_ID", calendar, "ADDED_HOLIDAYS", &addHolidays);
    getStringFieldFromTableforID("calendarios", "CALENDAR_ID", calendar, "REMOVED_HOLIDAYS", &deleteHolidays);
    QString fecha = QString("%1-%2-%3").arg(date.year()).arg(date.month()).arg(date.day());

    listaAdded=addHolidays.split(";");
    listaDeleted=deleteHolidays.split(";");

    if (listaAdded.contains(fecha))
        return true;

    index=listaDeleted.indexOf(fecha);
    if (index>0)
        listaDeleted.removeAt(index);

    listaAdded.append(fecha);

    query.prepare("UPDATE calendarios SET REMOVED_HOLIDAYS=:holidays WHERE CALENDAR_ID=:calendar");
    query.bindValue ( ":holidays", listaDeleted.join(";"));
    query.bindValue ( ":calendar", calendar);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query.lastError().text());
        return false;
    }
    query.prepare("UPDATE calendarios SET ADDED_HOLIDAYS=:holidays WHERE CALENDAR_ID=:calendar");
    query.bindValue ( ":holidays", listaAdded.join(";"));
    query.bindValue ( ":calendar", calendar);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query.lastError().text());
        return false;
    }

    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoFijaciones(QString indice, QDate fechaFijacion, double value)
{
    QSqlQuery query;


    query.prepare("INSERT INTO fijaciones (INDICE, FECHA, VALOR) "
                  "VALUES (:indice, :fecha, :valor)");
    query.bindValue(":indice", indice);
    query.bindValue(":fecha", fechaFijacion);
    query.bindValue(":valor", value);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }
    commit();
    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoTiposCambio(QString domestica, QString foranea, double quote)

{
    QSqlQuery query;

    query.prepare("INSERT INTO tipos_cambio (DOMESTIC_ID, FOREIGN_ID, QUOTE) VALUES (:domestica, :foranea, :quote)");
    query.bindValue(":domestica", domestica);
    query.bindValue(":foranea", foranea);
    query.bindValue(":quote", quote);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }
    commit();
    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoDepositos(QString id, QString currency, int plazo, QString periodo, double quote)
{

    QSqlQuery queryProducto;
    queryProducto.prepare("INSERT INTO productos (ID, TIPO, CURRENCY_ISO, QUOTE) "
                  "VALUES (:id, :tipo, :currency, :quote)");
    queryProducto.bindValue(":id", id);
    queryProducto.bindValue(":tipo", "Deposito");
    queryProducto.bindValue(":currency", currency);
    queryProducto.bindValue(":quote", quote);

    queryProducto.exec();
    if (queryProducto.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",queryProducto.lastError().text());
        return false;
    }
    QSqlQuery query;

    query.prepare("INSERT INTO depositos (ID, PLAZO, PERIODO) "
                  "VALUES (:id, :plazo, :periodo)");
    query.bindValue(":id", id);
    query.bindValue(":plazo", plazo);
    query.bindValue(":periodo", periodo);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }
    commit();
    QMessageBox::information(0,"Insercion en Base de Datos","Deposito insertado con exito");

    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoDivisas(QString iso, QString nombre, QString indice, int spot, QString calendario)
{

    QSqlQuery query;

    query.prepare("INSERT INTO divisas (CURRENCY_ISO, NAME, INDICE, SPOT_SCHEDULE, CALENDAR_ID) "
                  "VALUES (:iso, :name, :indice, :spot, :calendario)");
    query.bindValue(":iso", iso);
    query.bindValue(":name", nombre);
    query.bindValue(":indice", indice);
    query.bindValue(":spot", spot);
    query.bindValue(":calendario", calendario);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }
    commit();
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoFuturos(QString id, QString currency, int meses, QString immdate, double quote)
{

    QSqlQuery queryProducto;
    queryProducto.prepare("INSERT INTO productos (ID, TIPO, CURRENCY_ISO, QUOTE) "
                  "VALUES (:id, :tipo, :currency, :quote)");
    queryProducto.bindValue(":id", id);
    queryProducto.bindValue(":tipo", "Futuro");
    queryProducto.bindValue(":currency", currency);
    queryProducto.bindValue(":quote", quote);

    queryProducto.exec();
    if (queryProducto.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",queryProducto.lastError().text());
        return false;
    }

    QSqlQuery query;

    query.prepare("INSERT INTO futuros (ID, MONTHS, IMM_DATE) "
                  "VALUES (:id, :meses, :imm)");
    query.bindValue(":id", id);
    query.bindValue(":meses", meses);
    query.bindValue(":imm",immdate);

    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }
    commit();
    QMessageBox::information(0,"Insercion en Base de Datos","Futuro insertado con exito");

    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoFras(QString id, QString currency, int mesesStart, int mesesEnd, double quote)
{
    QSqlQuery queryProducto;
    queryProducto.prepare("INSERT INTO productos (ID, TIPO, CURRENCY_ISO, QUOTE) "
                  "VALUES (:id, :tipo, :currency, :quote)");
    queryProducto.bindValue(":id", id);
    queryProducto.bindValue(":tipo", "Fra");
    queryProducto.bindValue(":currency", currency);
    queryProducto.bindValue(":quote", quote);

    queryProducto.exec();
    if (queryProducto.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",queryProducto.lastError().text());
        return false;
    }
    QSqlQuery query;

    query.prepare("INSERT INTO fras (ID, MONTHSTOSTART, MONTHSTOEND) "
                  "VALUES (:id, :mesesstart, :mesesend)");
    query.bindValue(":id", id);
    query.bindValue(":mesesstart", mesesStart);
    query.bindValue(":mesesend", mesesEnd);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }
    commit();
    QMessageBox::information(0,"Insercion en Base de Datos","Fra insertado con exito");

    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoSwaps(QString id, QString currency, int plazo, QString periodo, QString frecuenciaFija, QString convFest, QString convCal, QString indice, double quote)
{

    QSqlQuery queryProducto;
    queryProducto.prepare("INSERT INTO productos (ID, TIPO, CURRENCY_ISO, QUOTE) "
                  "VALUES (:id, :tipo, :currency, :quote)");
    queryProducto.bindValue(":id", id);
    queryProducto.bindValue(":tipo", "Swap");
    queryProducto.bindValue(":currency", currency);
    queryProducto.bindValue(":quote", quote);

    queryProducto.exec();
    if (queryProducto.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",queryProducto.lastError().text());
        return false;
    }
    QSqlQuery query;
    query.prepare("INSERT INTO swaps (ID, PLAZO, PERIODO, FIXED_BUS_CONVENTION, FIXED_CAL_CONVENTION,  FIXEDLEG_FRECUENCY, FLOATING_INDEX) "
                  "VALUES (:id, :plazo, :periodo, :convfest, :convcal, :frecuenciafija, :indice)");
    query.bindValue(":id", id);
    query.bindValue(":plazo", plazo);
    query.bindValue(":periodo", periodo);
    query.bindValue(":convfest", convFest);
    query.bindValue(":convcal", convCal);
    query.bindValue(":frecuenciafija", frecuenciaFija);
    query.bindValue(":indice", indice);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }
    commit();
    QMessageBox::information(0,"Insercion en Base de Datos","Swap insertado con exito");


    return true;

}
//-----------------------------------------------------------------------------
bool DBHandler::insertIntoCurvas(QString curvaID, QString divisa, double tolerancia, QString convencionCal, QStringList productos)
{
    QSqlQuery queryCurva;
    queryCurva.prepare("INSERT INTO curvas(ID, CURRENCY_ISO, TOLERANCIA, CONVENCION_CALENDARIO) "
                  "VALUES (:id, :divisa, :tolerancia, :convencionCal)");
    queryCurva.bindValue(":id", curvaID);
    queryCurva.bindValue(":divisa", divisa);
    queryCurva.bindValue(":tolerancia", tolerancia);
    queryCurva.bindValue(":convencionCal", convencionCal);

    queryCurva.exec();
    if (queryCurva.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",queryCurva.lastError().text());
        return false;
    }
    QSqlQuery query;
    int numProductos=productos.count();
    for (int i=0; i<numProductos; i++)
    {
        query.prepare("INSERT INTO curva_producto (ID_CURVA, ID_PRODUCTO) "
                      "VALUES (:idcurva, :idproducto)");
        query.bindValue(":idcurva", curvaID);
        query.bindValue(":idproducto", productos.at(i));
        query.exec();
        if (query.lastError().number()>0)
        {
            QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
            rollback();
            return false;
        }
    }
    commit();
    QMessageBox::information(0,"Insercion en Base de Datos","Curva insertada con exito");

    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoIndices(QString indiceID, QString indiceEstandar, int plazo, QString periodo)
{
    QSqlQuery query;
    query.prepare("INSERT INTO indices(ID, INDICE_ESTANDAR, PLAZO, PERIODO) "
                  "VALUES (:id, :indEst, :plazo, :periodo)");
    query.bindValue(":id", indiceID);
    query.bindValue(":indEst", indiceEstandar);
    query.bindValue(":plazo", plazo);
    query.bindValue(":periodo", periodo);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }
    commit();
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoDeals(QString product, double nominal, QDate settlementDate, QDate maturityDate, QString currency, QString type, double spread, int *id)
{

    QSqlQuery query;
    int dealID=0;

    if (*id==0)
    {
        //Buscamos el máximo identificador en deals para usarlo+1
        query.prepare("SELECT MAX(ID) from deals");
        query.exec();
        if (query.lastError().number()>0)
        {
            QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
            rollback();
            return false;
        }
        query.next();
        dealID=query.value(0).toInt();

        dealID++;
        *id=dealID;
    }
    else
        dealID=*id;

//     PRODUCT	varchar(50)	latin1_spanish_ci		No	Ninguna		  Cambiar	  Eliminar	 Más
//     3	SETTLEMENT	date			No	Ninguna		  Cambiar	  Eliminar	 Más
//     4	MATURITY	date			No	Ninguna		  Cambiar	  Eliminar	 Más
//     5	CURRENCY_ISO	varchar(3)	latin1_spanish_ci		No	Ninguna		  Cambiar	  Eliminar	 Más
//     6	NOMINAL	int(15)		UNSIGNED	No	Ninguna		  Cambiar	  Eliminar	 Más
//     7	TYPE	enum('Pagador', 'Recibidor', 'Put', 'Call')	latin1_spanish_ci		No	Ninguna		  Cambiar	  Eliminar	 Más
//     8	SPREAD

    QString settlement = QString("%1-%2-%3").arg(settlementDate.year()).arg(settlementDate.month()).arg(settlementDate.day());
    QString maturity = QString("%1-%2-%3").arg(maturityDate.year()).arg(maturityDate.month()).arg(maturityDate.day());
    query.prepare("INSERT INTO deals (ID, PRODUCT, SETTLEMENT, MATURITY, CURRENCY_ISO, NOMINAL, TYPE, SPREAD) "
                  "VALUES (:id, :producto, :settlement, :maturity, :currency_iso, :nominal, :type, :spread)");
    query.bindValue(":id", dealID);
    query.bindValue(":producto", product);
    query.bindValue(":settlement", settlement);
    query.bindValue(":maturity", maturity);
    query.bindValue(":currency_iso", currency);
    query.bindValue(":nominal", nominal);
    query.bindValue(":type", type);
    query.bindValue(":spread", spread);

    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }

    return true;
}
//-----------------------------------------------------------------------------
bool DBHandler::insertIntoIrs(int isFwdStart, QDate fwdStart, int dealID)
{
    QSqlQuery query;
    QString fwdStartDate = QString("%1-%2-%3").arg(fwdStart.year()).arg(fwdStart.month()).arg(fwdStart.day());
    query.prepare("INSERT INTO irs (ID, IS_FWDSTART, FWDSTART) "
                  "VALUES (:id, :isFwd, :fwdStart)");
    query.bindValue(":id", dealID);
    query.bindValue(":isFwd", isFwdStart);
    query.bindValue(":fwdStart", fwdStartDate);

    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }

    return true;
}
//-----------------------------------------------------------------------------
bool DBHandler::insertIntoEuropeanOption(QString tipoSubyacente, QString subyacente, double strike, double tipoLibre, double volatilidad, QString convencionCalendario, int dealID)
{
    QSqlQuery query;
    query.prepare("INSERT INTO european_option (ID, TIPO_SUBYACENTE, SUBYACENTE, STRIKE, TIPO_LIBRE, VOLATILIDAD, CONVENCION_CALENDARIO) "
                  "VALUES (:id, :tipoSub, :sub, :strike, :tipo_libre, :volatilidad, :calendar)");
    query.bindValue(":id", dealID);
    query.bindValue(":tipoSub", tipoSubyacente);
    query.bindValue(":sub", subyacente);
    query.bindValue(":strike", strike);
    query.bindValue(":tipo_libre", tipoLibre);
    query.bindValue(":volatilidad", volatilidad);
    query.bindValue(":calendar", convencionCalendario);

    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }

    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoLeg( QString frequency, QString businessConv, QString calConv, QString curveName, QString legType, int *id)
{
    QSqlQuery query;

    //Primero buscamos el máximo identificador para usarlo+1
    query.prepare("SELECT MAX(ID) from leg");
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }
    query.next();
    int legID=query.value(0).toInt();
    legID++;
    *id=legID;

    query.prepare("INSERT INTO leg (ID, FRECUENCIA, TIPO, CONVENCION, CONTADOR, CURVA_ID) "
                  "VALUES (:id, :frecuencia, :tipo, :convencion, :contador, :curva)");
    query.bindValue(":id", legID);
    query.bindValue(":frecuencia", frequency);
    query.bindValue(":tipo", legType);
    query.bindValue(":convencion", businessConv);
    query.bindValue(":contador", calConv);
    query.bindValue(":curva", curveName);

    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }

    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoFixedLeg(double fixedRate, int legID)
{

    QSqlQuery query;

    query.prepare("INSERT INTO leg_fixed (ID, TIPO_FIJO) "
                  "VALUES (:legID, :tipoFijo)");
    query.bindValue(":legID", legID);
    query.bindValue(":tipoFijo", fixedRate);

    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }

    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertIntoFloatingLeg(QString indexName, int legID)
{
    QSqlQuery query;
    query.prepare("INSERT INTO leg_floating (ID, ID_FLOATING_INDEX) "
                  "VALUES (:legID, :index)");

    query.bindValue(":legID", legID);
    query.bindValue(":index", indexName);

    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }

    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertIRS(double nominal, QDate settlementDate, QDate maturityDate, QString currency, QString rol, double spread,
               bool isForward, QDate fwdStart, QString fixedFrequency, QString fixedBusinessConv,
               QString fixedCalConv, double fixedRate, QString discountCurveName, QString floatingFrequency,
               QString floatingBusinessConv, QString floatingCalConv, QString indexName, QString forwardCurveName, int *id)
{
    bool bret=false;
    int dealID=0;
    int fixLeg=0;
    int floatLeg=0;
    //primero insertamos en deals
    if (*id)
        dealID=*id;
    bret=insertIntoDeals("IRS", nominal, settlementDate, maturityDate, currency, rol, spread, &dealID);
    if (!bret)
        return bret;
    //despues en irs
    bret=insertIntoIrs(isForward, fwdStart, dealID);
    if (!bret)
    {
        rollback();
        return bret;
    }
    //despues leg
    bret=insertIntoLeg( fixedFrequency, fixedBusinessConv, fixedCalConv, discountCurveName, "Fija", &fixLeg);
    if (!bret)
    {
        rollback();
        return bret;
    }
    bret=insertIntoLeg( floatingFrequency, floatingBusinessConv, floatingCalConv, forwardCurveName, "Flotante", &floatLeg);
    if (!bret)
    {
        rollback();
        return bret;
    }

    //despues fixedleg
    bret=insertIntoFixedLeg(fixedRate, fixLeg);
    if (!bret)
    {
        rollback();
        return bret;
    }

    //continuamos con floating leg
     bret=insertIntoFloatingLeg(indexName, floatLeg);
    if (!bret)
    {
        rollback();
        return bret;
    }

    //finalizamos estableciendo la relacion entre las patas y el deal
    QSqlQuery query;
    query.prepare("INSERT INTO deals_leg (ID_DEAL, ID_LEG) "
                  "VALUES (:idDeal, :idLeg)");
    query.bindValue(":idDeal", dealID);
    query.bindValue(":idLeg", fixLeg);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }
    query.prepare("INSERT INTO deals_leg (ID_DEAL, ID_LEG) "
                  "VALUES (:idDeal, :idLeg)");
    query.bindValue(":idDeal", dealID);
    query.bindValue(":idLeg", floatLeg);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        rollback();
        return false;
    }
    commit();
    *id=dealID;
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::insertEuropeanOption(double nominal, QDate settlementDate, QDate maturityDate,
                QString currency, QString tipoOpcion, QString tipoSubyacente, QString subyacente,
                double strike, double tipoLibre, double volatilidad, QString convencionCalendario, int *id)
{
    bool bret=false;
    int dealID=0;

    //primero insertamos en deals
    if (*id)
        dealID=*id;
    bret=insertIntoDeals("European Option", nominal, settlementDate, maturityDate, currency, tipoOpcion, 0, &dealID);
    if (!bret)
        return bret;
    //despues en European Option
    bret=insertIntoEuropeanOption(tipoSubyacente, subyacente, strike, tipoLibre, volatilidad, convencionCalendario, dealID);
    if (!bret)
    {
        rollback();
        return bret;
    }
    commit();
    *id=dealID;
    return true;
}

//-----------------------------------------------------------------------------
QSqlQuery *DBHandler::getFamilies()
{
    QSqlQuery *query=new QSqlQuery("SELECT ID FROM familias");
    return query;
}

//-----------------------------------------------------------------------------
QSqlQuery *DBHandler::getFamilyProducts(QString family)
{
    QSqlQuery *query = new QSqlQuery;
    query->prepare("SELECT DISTINCT ID_PRODUCTO FROM familia_tipo_producto where familia_tipo_producto.ID_FAMILIA=:value");
    query->bindValue(":value", family);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
    }

    return query;
}

//-----------------------------------------------------------------------------
QSqlQuery *DBHandler::getDeals(QString product)
{
    QSqlQuery *query = new QSqlQuery;
    query->prepare("SELECT * FROM deals WHERE PRODUCT=:producto");
    query->bindValue(":PRODUCT", product);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
    }
    return query;
}

//-----------------------------------------------------------------------------
bool DBHandler::getDeals(QStringList *lista)
{
    if (!lista)
        lista=new QStringList();

    QSqlQuery *query = new QSqlQuery;
    query->prepare("SELECT ID FROM deals");
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    while (query->next())
        lista->append(query->value(0).toString());
    delete query;
    return true;
}

//-----------------------------------------------------------------------------
void DBHandler::getDeals(QTableWidget *tabla, QString product)
{
    QSqlQuery *query;
    QStringList *columnas=this->getColumnsListFromTable("deals");
    int numColumns=columnas->count();
    for (int i=tabla->rowCount()-1; i >= 0; --i)
        tabla->removeRow(i);
    tabla->setColumnCount(numColumns);
    tabla->setHorizontalHeaderLabels(*columnas);
    delete columnas;
    query=this->getDeals(product);
    int actualRow=0;
    tabla->setRowCount(query->size());
    while (query->next())
    {
        for (int actualColumn=0; actualColumn<numColumns; actualColumn++)
        {
            tabla->setItem(actualRow, actualColumn, new QTableWidgetItem(query->value(actualColumn).toString()));
        }
        actualRow++;
    }
    tabla->resizeColumnsToContents();

    if (tabla->rowCount())
        tabla->selectRow(0);

    delete query;

}

//------------------------------------------------------------------------------
void DBHandler::getOrderedDeals(QTableWidget *tabla)
{

    QSqlQuery query;
    QString tituloTabla;
    int numColumns=5;
    tituloTabla.append("CONTRATO;TIPO DE PRODUCTO;NOMINAL;DIVISA;VENCIMIENTO");
    tabla->setColumnCount(numColumns);
    tabla->setHorizontalHeaderLabels(tituloTabla.split(";"));

    for (int i=tabla->rowCount()-1; i >= 0; --i)
        tabla->removeRow(i);


    query.exec("SELECT ID,PRODUCT,NOMINAL,CURRENCY_ISO,MATURITY FROM deals ORDER BY MATURITY ASC");
    int actualRow=0;
    tabla->setRowCount(query.size());
    while (query.next())
    {
        for (int actualColumn=0; actualColumn<numColumns; actualColumn++)
        {
            tabla->setItem(actualRow, actualColumn, new QTableWidgetItem(query.value(actualColumn).toString()));
        }
        actualRow++;
    }
    tabla->resizeColumnsToContents();

    if (tabla->rowCount())
        tabla->selectRow(0);

}


//-----------------------------------------------------------------------------
bool DBHandler::getIntFieldFromTableforID(QString table, QString primary, QString id, QString field, int *ret)
{
    QSqlQuery *query= new QSqlQuery;
    QString queryTXT("SELECT ");
    queryTXT.append(field);
    queryTXT.append(" FROM ");
    queryTXT.append(table);
    queryTXT.append(" WHERE ");
    queryTXT.append(primary);
    queryTXT.append("=:id");
    query->prepare(queryTXT);
    query->bindValue(":id", id);

    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    query->next();
    *ret=query->value(0).toInt();
    delete query;
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::getDoubleFieldFromTableforID(QString table, QString primary, QString id, QString field, double *ret)
{
    QSqlQuery *query= new QSqlQuery;
    QString queryTXT("SELECT ");
    queryTXT.append(field);
    queryTXT.append(" FROM ");
    queryTXT.append(table);
    queryTXT.append(" WHERE ");
    queryTXT.append(primary);
    queryTXT.append("=:id");
    query->prepare(queryTXT);
    query->bindValue(":id", id);

    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    query->next();
    *ret=query->value(0).toDouble();
    delete query;
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::getStringFieldFromTableforID(QString table, QString primary, QString id, QString field, QString *ret)
{
    QSqlQuery *query= new QSqlQuery;
    QString queryTXT("SELECT ");
    queryTXT.append(field);
    queryTXT.append(" FROM ");
    queryTXT.append(table);
    queryTXT.append(" WHERE ");
    queryTXT.append(primary);
    queryTXT.append("=:id");
    query->prepare(queryTXT);
    query->bindValue(":id", id);

    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    query->next();
    *ret=query->value(0).toString();
    delete query;
    return true;
}

//-----------------------------------------------------------------------------
unsigned int DBHandler::getCurrencySpotSchedule(QString currency)
{
    QSqlQuery *query= new QSqlQuery;
    int iret=-1;
    int index=0;

    query->prepare("SELECT * FROM divisas WHERE CURRENCY_ISO=:currency");
    query->bindValue(":currency", currency);

    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return 0;
    }
    else
    {
        while (query->next())
        {
            index=query->record().indexOf("SPOT_SCHEDULE");
            iret=query->value(index).toInt();
            delete query;
            return iret;
        }
    }
    return 0;

}

//-----------------------------------------------------------------------------
QSqlQuery *DBHandler::getProducto(QString tipoProducto, QString idProducto)
{
    QSqlQuery *query= new QSqlQuery;
    QString queryTXT="SELECT d.*, p.CURRENCY_ISO, p.QUOTE from ";
    queryTXT.append(tipoProducto);
    queryTXT.append(" d INNER JOIN productos p ON p.ID = d.ID where p.ID=:id");
    query->prepare(queryTXT);
    query->bindValue(":id", idProducto);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
    }
    return query;
}
//-----------------------------------------------------------------------------
QStringList *DBHandler::getCurvesContainingDivisa(QString divisa)
{
    QStringList *lista=new QStringList();
    QSqlQuery *query= new QSqlQuery;
    QString queryTXT="SELECT ID FROM curvas where CURRENCY_ISO=:divisa";
    query->prepare(queryTXT);
    query->bindValue(":divisa", divisa);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return lista;
    }
    while (query->next())
    {
        lista->append(query->value(0).toString());
    }

    delete query;
    return lista;
}

//-----------------------------------------------------------------------------
QStringList *DBHandler::getDealsContainingDivisa(QString divisa)
{
    QStringList *lista=new QStringList();
    QSqlQuery *query= new QSqlQuery;
    QString queryTXT="SELECT ID FROM deals where CURRENCY_ISO=:divisa";
    query->prepare(queryTXT);
    query->bindValue(":divisa", divisa);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return lista;
    }
    while (query->next())
    {
        lista->append(query->value(0).toString());
    }

    delete query;
    return lista;
}

//-----------------------------------------------------------------------------
QStringList *DBHandler::getProductsContainingDivisa(QString divisa)
{
    QStringList *lista=new QStringList();
    QSqlQuery *query= new QSqlQuery;
    QString queryTXT="SELECT ID FROM productos where CURRENCY_ISO=:divisa";
    query->prepare(queryTXT);
    query->bindValue(":divisa", divisa);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return lista;
    }
    while (query->next())
    {
        lista->append(query->value(0).toString());
    }

    delete query;
    return lista;
}

//-----------------------------------------------------------------------------
QStringList *DBHandler::getCurvesContainingProduct(QString producto)
{
    QStringList *lista=new QStringList();
    QSqlQuery *query= new QSqlQuery;
    QString queryTXT="SELECT curva_producto.ID_CURVA FROM curva_producto where curva_producto.ID_PRODUCTO=:producto";
    query->prepare(queryTXT);
    query->bindValue(":producto", producto);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return lista;
    }
    while (query->next())
    {
        lista->append(query->value(0).toString());
    }

    delete query;
    return lista;
}

//-----------------------------------------------------------------------------
QStringList *DBHandler::getDealsContainingCurve(QString curve)
{
    QStringList *lista=new QStringList();
    QSqlQuery *query= new QSqlQuery;
    query->prepare("SELECT ID FROM leg where CURVA_ID=:curva");
    query->bindValue(":curva", curve);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return lista;
    }
    while (query->next())
    {
        lista->append(query->value(0).toString());
    }

    delete query;
    return lista;
}

//-----------------------------------------------------------------------------
QStringList *DBHandler::getDealsContainingIndex(QString indice)
{
    QStringList *lista=new QStringList();
    QSqlQuery *query= new QSqlQuery;
    query->prepare("SELECT ID FROM leg_floating where ID_FLOATING_INDEX=:indice");
    query->bindValue(":indice", indice);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return lista;
    }
    while (query->next())
    {
        lista->append(query->value(0).toString());
    }

    delete query;
    return lista;
}

//-----------------------------------------------------------------------------
bool DBHandler::updateQuote(QString producto, double newQuote)
{
    QSqlQuery *query= new QSqlQuery;

    QString queryTXT="UPDATE productos SET quote = :valor WHERE productos.ID = :producto";
    query->prepare(queryTXT);
    query->bindValue(":valor", newQuote);
    query->bindValue(":producto", producto);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    delete query;
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::updateTipoCambio(QString domestic, QString foreign, double tipo)
{
    QSqlQuery *query= new QSqlQuery;

    query->prepare("UPDATE tipos_cambio SET QUOTE = :tipo WHERE DOMESTIC_ID = :domestic AND FOREIGN_ID = :foreign");
    query->bindValue(":tipo", tipo);
    query->bindValue(":domestic", domestic);
    query->bindValue(":foreign", foreign);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    delete query;
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::updateDefaultCurrency(QString defaultCurrency)
{
    QSqlQuery *query= new QSqlQuery;

    query->prepare("UPDATE divisa_defecto SET CURRENCY_ISO = :defaultCurrency");
    query->bindValue(":defaultCurrency", defaultCurrency);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    delete query;
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::getDefaultCurrency(QString *defaultCurrency)
{
    QSqlQuery *query= new QSqlQuery;

    query->prepare("SELECT CURRENCY_ISO FROM divisa_defecto");
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    query->next();
    *defaultCurrency=query->value(0).toString();
    delete query;
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::deleteProducto(QString producto)
{
    QSqlQuery *queryTipo= new QSqlQuery;
    QSqlQuery *query= new QSqlQuery;
    int index=-1;
    QString tipo;

    //Primero recogemos el tipo de producto para borrar despues la tabla correspondiente
    queryTipo->prepare("SELECT productos.tipo from productos where productos.ID = :producto");
    queryTipo->bindValue(":producto",producto);
    queryTipo->exec();
    if (queryTipo->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",queryTipo->lastError().text());
        delete queryTipo;
        return false;
    }

    while (queryTipo->next())
    {
        index=queryTipo->record().indexOf("TIPO");
        tipo=queryTipo->value(index).toString();
    }

    delete queryTipo;

    //Borramos el producto de la tabla custom de producto
    if (tipo=="Deposito")
        query->prepare("DELETE from depositos where ID = :producto");
    else if (tipo=="Futuro")
        query->prepare("DELETE from futuros where ID = :producto");
    else if (tipo=="Fra")
        query->prepare("DELETE from fras where ID = :producto");
    else
        query->prepare("DELETE from swaps where ID = :producto");

    query->bindValue(":producto", producto);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }

    //Borramos el producto de la tabla de productos
    query->prepare("DELETE from productos where productos.ID = :producto");
    query->bindValue(":producto", producto);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }

    delete query;
    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::deleteCurva(QString curva)
{
    QSqlQuery *query= new QSqlQuery;
    QString tipo;

    //Primero borramos de la tabla curva_producto
    query->prepare("delete from curva_producto where curva_producto.ID_CURVA=:curva");
    query->bindValue(":curva",curva);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }

    //Borramos la entrada de la tabla de curvas
    query->prepare("delete from curvas where ID = :curva");
    query->bindValue(":curva", curva);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    delete query;
    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::deleteIndice(QString indice)
{
    QSqlQuery *query= new QSqlQuery;

    query->prepare("delete from indices where ID=:indice");
    query->bindValue(":indice",indice);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }

    delete query;
    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::deleteTipoCambio(QString domestica, QString foranea)
{
    QSqlQuery *query= new QSqlQuery;

    query->prepare("delete from tipos_cambio where DOMESTIC_ID=:domestica and FOREIGN_ID=:foranea");
    query->bindValue(":domestica",domestica);
    query->bindValue(":foranea",foranea);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }

    delete query;
    return true;
}

//-----------------------------------------------------------------------------
double DBHandler::getTipoCambio(QString domestica, QString foranea)
{

    double dret=0.;

    if (domestica==foranea)
        return 1.;

    QSqlQuery *query= new QSqlQuery;
    query->prepare("select QUOTE from tipos_cambio where DOMESTIC_ID=:domestica and FOREIGN_ID=:foranea");
    query->bindValue(":domestica",domestica);
    query->bindValue(":foranea",foranea);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    query->next();

    if (query->size()) //Si hay una entrada
        dret=query->value(0).toDouble();

    //En caso contrario buscamos la inversa
    query->prepare("select QUOTE from tipos_cambio where DOMESTIC_ID=:domestica and FOREIGN_ID=:foranea");
    query->bindValue(":domestica",foranea);
    query->bindValue(":foranea",domestica);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }
    query->next();

    if (query->size()) //Si hay una entrada
    {
        dret=query->value(0).toDouble();
        //Y recordamos que hay que invertirlo
        if (dret)
            dret=1/dret;
    }

    delete query;
    return dret;
}

//-----------------------------------------------------------------------------
bool DBHandler::deleteHolidayFromCalendario(QString calendar, QDate date)
{
    QSqlQuery query;
    QStringList listaDeleted;
    QStringList listaAdded;
    QString addHolidays, deleteHolidays;
    int index=-1;

    getStringFieldFromTableforID("calendarios", "CALENDAR_ID", calendar, "ADDED_HOLIDAYS", &addHolidays);
    getStringFieldFromTableforID("calendarios", "CALENDAR_ID", calendar, "REMOVED_HOLIDAYS", &deleteHolidays);
    QString fecha = QString("%1-%2-%3").arg(date.year()).arg(date.month()).arg(date.day());

    listaAdded=addHolidays.split(";");
    listaDeleted=deleteHolidays.split(";");

    if (listaDeleted.contains(fecha))
        return true;

    index=listaAdded.indexOf(fecha);
    if (index>0)
        listaAdded.removeAt(index);

    listaDeleted.append(fecha);

    query.prepare("UPDATE calendarios SET REMOVED_HOLIDAYS=:holidays WHERE CALENDAR_ID=:calendar");
    query.bindValue ( ":holidays", listaDeleted.join(";"));
    query.bindValue ( ":calendar", calendar);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query.lastError().text());
        return false;
    }
    query.prepare("UPDATE calendarios SET ADDED_HOLIDAYS=:holidays WHERE CALENDAR_ID=:calendar");
    query.bindValue ( ":holidays", listaAdded.join(";"));
    query.bindValue ( ":calendar", calendar);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query.lastError().text());
        return false;
    }
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::deleteDivisa(QString divisa)
{
    QSqlQuery *query= new QSqlQuery;

    query->prepare("delete from divisas where CURRENCY_ISO=:divisa");
    query->bindValue(":divisa",divisa);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }

    delete query;
    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::deleteFijacion(QString indice, QString fechaFijacion)
{
    QSqlQuery *query= new QSqlQuery;

    query->prepare("delete from fijaciones where INDICE=:indice and FECHA=:fecha");
    query->bindValue(":indice",indice);
    query->bindValue(":fecha",fechaFijacion);
    query->exec();
    if (query->lastError().number()>0)
    {
        QMessageBox::critical(0,"Database Error",query->lastError().text());
        delete query;
        return false;
    }

    delete query;
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::deleteDeal(int dealID)
{
    QSqlQuery queryLeg;
    QSqlQuery query;
    int idLeg;
    QString tipoLeg;
    QString producto;
    queryLeg.prepare("SELECT dl.ID_LEG, l.TIPO FROM deals_leg dl INNER JOIN leg l ON dl.ID_LEG = l.ID where ID_DEAL=:dealID");
    queryLeg.bindValue(":dealID", dealID);
    queryLeg.exec();
    if (queryLeg.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",queryLeg.lastError().text());
        return false;
    }
    query.prepare("DELETE FROM deals_leg where ID_DEAL=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }
    while (queryLeg.next())
    {
        idLeg=queryLeg.value(0).toInt();
        tipoLeg=queryLeg.value(1).toString();
        QString queryTXT("DELETE FROM ");
        if (tipoLeg=="Fija")
            queryTXT.append(" leg_fixed ");
        else
            queryTXT.append(" leg_floating ");
        queryTXT.append("where ID=");
        queryTXT.append(QString("%1").arg(idLeg));
        query.prepare(queryTXT);
        query.exec();
        if (query.lastError().number()>0)
        {
            QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
            return false;
        }
        query.prepare("DELETE FROM leg where ID=:id");
        query.bindValue(":id",idLeg);
        query.exec();
        if (query.lastError().number()>0)
        {
            QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
            return false;
        }
    }
    query.prepare("SELECT PRODUCT FROM deals where ID=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }
    query.next();
    producto=query.value(0).toString();
    producto=producto.toLower();
    producto.replace(" ","_");
    QString queryTXT("DELETE FROM ");
    queryTXT.append(producto);
    queryTXT.append(" WHERE ID=");
    queryTXT.append(QString("%1").arg(dealID));
    query.prepare(queryTXT);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }
    query.prepare("DELETE FROM deals WHERE ID=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }
    commit();
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::getIRS(int dealID, double *nominal, QDate *settlement, QDate *maturity,
                QString *divisa, QString *rol, double *spread,
                bool *isFwd, QDate *fwdStart,
                QString *frecuenciaFija, QString *convencionFestivosFija,
                QString *convencionCalendarioFija, double *tipoFijo,
                QString *curvaDescuento, QString *frecuenciaFlotante,
                QString *convencionFestivosFlotante, QString *convencionCalendarioFlotante,
                QString *indice, QString *curvaForward)
{
    //Cargamos lo conscerniente a Deal
    QSqlQuery query, queryLeg;
    int legID=-1;
    QString frecuencia, legType, convencion, contador, curva;

    query.prepare("SELECT * from deals where ID=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    query.next();
    if (query.value(1).toString()!="IRS")
        return false;
    *settlement=query.value(2).toDate();
    *maturity=query.value(3).toDate();

    *divisa=query.value(4).toString();
    *nominal=query.value(5).toDouble();
    *rol=query.value(6).toString();
    *spread=query.value(7).toDouble();

    //Cargamos aquello específico de IRS
    query.prepare("SELECT * from irs where ID=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    query.next();
    if (query.value(1).toInt())
    {
        *isFwd=true;
        *fwdStart=query.value(2).toDate();
    }
    else
        *isFwd=false;


    //Cargamos lo conscerniente a cada pata
    query.prepare("SELECT l.ID, l.FRECUENCIA, l.TIPO, l.CONVENCION, l.CONTADOR, l.CURVA_ID  from leg l INNER JOIN deals_leg dl ON l.ID = dl.ID_LEG where dl.ID_DEAL=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    while (query.next())
    {
        legID = query.value(0).toInt();
        frecuencia = query.value(1).toString();
        legType = query.value(2).toString();
        convencion = query.value(3).toString();
        contador = query.value(4).toString();
        curva = query.value(5).toString();
        if (legType=="Flotante")
        {
            queryLeg.prepare("SELECT ID_FLOATING_INDEX from leg_floating where ID=:id");
            queryLeg.bindValue(":id",legID);
            queryLeg.exec();
            if (queryLeg.lastError().number()>0)
            {
                QMessageBox::critical(0,"Error en Base de Datos",queryLeg.lastError().text());
                return false;
            }
            if (queryLeg.next())
            {
                *indice=queryLeg.value(0).toString();
                *frecuenciaFlotante=frecuencia;
                *convencionFestivosFlotante=convencion;
                *convencionCalendarioFlotante=contador;
                *curvaForward=curva;
            }
        }
        else
        {
            queryLeg.prepare("SELECT TIPO_FIJO from leg_fixed where ID=:id");
            queryLeg.bindValue(":id",legID);
            queryLeg.exec();
            if (queryLeg.lastError().number()>0)
            {
                QMessageBox::critical(0,"Error en Base de Datos",queryLeg.lastError().text());
                return false;
            }
            if (queryLeg.next())
            {
                *tipoFijo=queryLeg.value(0).toDouble();
                *frecuenciaFija=frecuencia;
                *convencionFestivosFija=convencion;
                *convencionCalendarioFija=contador;
                *curvaDescuento=curva;
            }
        }
    }
    return true;
}

//-----------------------------------------------------------------------------
bool DBHandler::getIRS(int dealID, QDoubleSpinBox *doubleSpinBoxNominal, QDateEdit *dateEditSettlement, QDateEdit *dateEditMaturity,
                QComboBox *comboBoxDivisa, QComboBox *comboBoxRol, QDoubleSpinBox *doubleSpinBoxSpread,
                QCheckBox *checkBoxIsFwd, QDateEdit *dateEditFwdStart,
                QComboBox *comboBoxFrecuenciaFija, QComboBox *comboBoxConvencionFestivosFija,
                QComboBox *comboBoxConvencionCalendarioFija, QDoubleSpinBox *doubleSpinBoxTipoFijo,
                QComboBox *comboBoxCurvaDescuento, QComboBox *comboBoxFrecuenciaFlotante,
                QComboBox *comboBoxConvencionFestivosFlotante, QComboBox *comboBoxConvencionCalendarioFlotante,
                QComboBox *comboBoxIndice, QComboBox *comboBoxCurvaForward)
{
    //Cargamos lo conscerniente a Deal
    QSqlQuery query, queryLeg;
    int index=-1, legID=-1;
    QString indice, frecuencia, legType, convencion, contador, curva;
    double tipoFijo=0.0;

    query.prepare("SELECT * from deals where ID=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    query.next();
    if (query.value(1).toString()!="IRS")
        return false;
    dateEditSettlement->setDate(query.value(2).toDate());
    dateEditMaturity->setDate(query.value(3).toDate());
    index = comboBoxDivisa->findText(query.value(4).toString());
    comboBoxDivisa->setCurrentIndex(index);
    doubleSpinBoxNominal->setValue(query.value(5).toDouble());
    index = comboBoxRol->findText(query.value(6).toString());
    comboBoxRol->setCurrentIndex(index);
    doubleSpinBoxSpread->setValue(query.value(7).toDouble());

    //Cargamos aquello específico de IRS
    query.prepare("SELECT * from irs where ID=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    query.next();
    if (query.value(1).toInt())
    {
        checkBoxIsFwd->setChecked(true);
        dateEditFwdStart->setDate(query.value(2).toDate());
    }
    else
        checkBoxIsFwd->setChecked(false);


    //Cargamos lo conscerniente a cada pata
    query.prepare("SELECT l.ID, l.FRECUENCIA, l.TIPO, l.CONVENCION, l.CONTADOR, l.CURVA_ID  from leg l INNER JOIN deals_leg dl ON l.ID = dl.ID_LEG where dl.ID_DEAL=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    while (query.next())
    {
        legID = query.value(0).toInt();
        frecuencia = query.value(1).toString();
        legType = query.value(2).toString();
        convencion = query.value(3).toString();
        contador = query.value(4).toString();
        curva = query.value(5).toString();
        if (legType=="Flotante")
        {
            queryLeg.prepare("SELECT ID_FLOATING_INDEX from leg_floating where ID=:id");
            queryLeg.bindValue(":id",legID);
            queryLeg.exec();
            if (queryLeg.lastError().number()>0)
            {
                QMessageBox::critical(0,"Error en Base de Datos",queryLeg.lastError().text());
                return false;
            }
            if (queryLeg.next())
            {
                indice=queryLeg.value(0).toString();
                index = comboBoxFrecuenciaFlotante->findText(frecuencia);
                comboBoxFrecuenciaFlotante->setCurrentIndex(index);
                index = comboBoxConvencionFestivosFlotante->findText(convencion);
                comboBoxConvencionFestivosFlotante->setCurrentIndex(index);
                index = comboBoxConvencionCalendarioFlotante->findText(contador);
                comboBoxConvencionCalendarioFlotante->setCurrentIndex(index);
                index = comboBoxIndice->findText(indice);
                comboBoxIndice->setCurrentIndex(index);
                index=comboBoxCurvaForward->findText(curva);
                comboBoxCurvaForward->setCurrentIndex(index);
            }
        }
        else
        {
            queryLeg.prepare("SELECT TIPO_FIJO from leg_fixed where ID=:id");
            queryLeg.bindValue(":id",legID);
            queryLeg.exec();
            if (queryLeg.lastError().number()>0)
            {
                QMessageBox::critical(0,"Error en Base de Datos",queryLeg.lastError().text());
                return false;
            }
            if (queryLeg.next())
            {
                tipoFijo=queryLeg.value(0).toDouble();
                doubleSpinBoxTipoFijo->setValue(tipoFijo);
                index = comboBoxFrecuenciaFija->findText(frecuencia);
                comboBoxFrecuenciaFija->setCurrentIndex(index);
                index = comboBoxConvencionFestivosFija->findText(convencion);
                comboBoxConvencionFestivosFija->setCurrentIndex(index);
                index = comboBoxConvencionCalendarioFija->findText(contador);
                comboBoxConvencionCalendarioFija->setCurrentIndex(index);
                index=comboBoxCurvaDescuento->findText(curva);
                comboBoxCurvaDescuento->setCurrentIndex(index);
            }

        }
    }
    return true;
}
//-----------------------------------------------------------------------------
bool DBHandler::getEuropeanOption( int dealID, double *nominal, QDate *settlement,
            QDate *maturity, QString *divisa, QString *tipoOpcion, QString *tipoSubyacente,
            QString *subyacente, double *spot, double *strike,double *tipoLibre, double *volatilidad, QString *convencionCalendario)
{
    double quote;

    //Cargamos lo conscerniente a Deal
    QSqlQuery query;

    query.prepare("SELECT * from deals where ID=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    query.next();
    if (query.value(1).toString()!="European Option")
        return false;

    *settlement=query.value(2).toDate();
    *maturity=query.value(3).toDate();
    *divisa=query.value(4).toString();
    *nominal=query.value(5).toDouble();
    *tipoOpcion=query.value(6).toString();


    //Cargamos aquello específico de European Option
    query.prepare("SELECT * from european_option where ID=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    query.next();
    (*tipoSubyacente)=query.value(1).toString();
    (*subyacente)=query.value(2).toString();
    getDoubleFieldFromTableforID("productos", "ID", *subyacente, "QUOTE", &quote);
    *spot=quote;
    *strike=query.value(3).toDouble();
    *tipoLibre=query.value(4).toDouble();
    *volatilidad=query.value(5).toDouble();

    *convencionCalendario=query.value(6).toString();
    return true;

}

//-----------------------------------------------------------------------------
bool DBHandler::getEuropeanOption( int dealID, QDoubleSpinBox *doubleSpinBoxNominal, QDateEdit *dateEditSettlement,
            QDateEdit *dateEditMaturity, QComboBox *comboBoxDivisa, QComboBox *comboBoxTipoOpcion, QComboBox *comboBoxTipoSubyacente,
            QComboBox *comboBoxSubyacente, QDoubleSpinBox *doubleSpinBoxSpot, QDoubleSpinBox *doubleSpinBoxStrike,
            QDoubleSpinBox *doubleSpinBoxTipoLibre, QDoubleSpinBox *doubleSpinBoxVolatilidad, QComboBox *comboBoxConvencionCalendario)
{
    double quote;

    //Cargamos lo conscerniente a Deal
    QSqlQuery query;
    int index=-1;

    query.prepare("SELECT * from deals where ID=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    query.next();
    if (query.value(1).toString()!="European Option")
        return false;
    dateEditSettlement->setDate(query.value(2).toDate());
    dateEditMaturity->setDate(query.value(3).toDate());
    index = comboBoxDivisa->findText(query.value(4).toString());
    comboBoxDivisa->setCurrentIndex(index);
    doubleSpinBoxNominal->setValue(query.value(5).toDouble());
    index = comboBoxTipoOpcion->findText(query.value(6).toString());
    comboBoxTipoOpcion->setCurrentIndex(index);

    //Cargamos aquello específico de European Option
    query.prepare("SELECT * from european_option where ID=:id");
    query.bindValue(":id",dealID);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
        return false;
    }

    query.next();
    index = comboBoxTipoSubyacente->findText(query.value(1).toString());
    comboBoxTipoSubyacente->setCurrentIndex(index);

    index = comboBoxSubyacente->findText(query.value(2).toString());
    comboBoxSubyacente->setCurrentIndex(index);
    getDoubleFieldFromTableforID("productos", "ID", comboBoxSubyacente->currentText(), "QUOTE", &quote);
    doubleSpinBoxSpot->setValue(quote);
    doubleSpinBoxStrike->setValue(query.value(3).toDouble());
    doubleSpinBoxTipoLibre->setValue(query.value(4).toDouble());
    doubleSpinBoxVolatilidad->setValue(query.value(5).toDouble());

    index = comboBoxConvencionCalendario->findText(query.value(6).toString());
    comboBoxConvencionCalendario->setCurrentIndex(index);

    return true;
}

//------------------------------------------------------------------------------
void DBHandler::addIndexFixings(QString indexName, boost::shared_ptr<QuantLib::IborIndex> index)
{
    using namespace QuantLib;

    QSqlQuery query;
    QString dbDate;
    QStringList lista;
    int year,month,day;
    double value;

    query.prepare("select FECHA, VALOR from fijaciones WHERE INDICE=:indice");
    query.bindValue(":indice",indexName);
    query.exec();
    if (query.lastError().number()>0)
    {
        QMessageBox::critical(0,"Error en Base de Datos",query.lastError().text());
    }
    while (query.next())
    {
        dbDate=query.value(0).toString();
        value=query.value(1).toDouble();
        lista=dbDate.split("-");

        year=lista.at(0).toInt();
        month=lista.at(1).toInt();
        day=lista.at(2).toInt();
        Date date=Date(day, (Month)month, year);
        index->addFixing(date, value);
    }
}

//-----------------------------------------------------------------------------
void DBHandler::exportDealsToFile(QString fileName)
{

    QStringList deals;
    QStringList listaEO, listaIRS;
    QString EO,IRS, text, producto;
    bool isFwd;
    double nominal=0., spot=0., strike=0., tipoLibre=0., volatilidad=0., spread=0., tipoFijo=0.;
    QDate settlement, maturity, fwdStart;
    QString divisa, tipoOpcion, tipoSubyacente, subyacente, convencionCalendario, convencionCalendarioFija, rol, frecuenciaFija, convencionFestivosFija;
    QString curvaDescuento, frecuenciaFlotante, convencionFestivosFlotante, convencionCalendarioFlotante, indice, curvaForward;

    if (fileName=="")
        return;

    getDeals(&deals);
    int numDeals=deals.count();

    for (int i=0; i<numDeals; i++)
    {
        getStringFieldFromTableforID("deals", "ID", deals.at(i), "PRODUCT", &producto);
        if (producto=="European Option")
        {
            getEuropeanOption( deals.at(i).toInt(), &nominal, &settlement, &maturity, &divisa, &tipoOpcion, &tipoSubyacente,
                                    &subyacente, &spot, &strike, &tipoLibre, &volatilidad, &convencionCalendario);

            EO=QString("%1;%2;%3;%4;%5;%6;%7;%8;%9;%10;%11;%12;%13").arg(deals.at(i)).arg(nominal).arg(settlement.toString()).arg(maturity.toString())
                    .arg(divisa).arg(tipoOpcion).arg(tipoSubyacente).arg(subyacente).arg(spot).arg(strike).arg(tipoLibre).arg(volatilidad).arg(convencionCalendario);
            listaEO.append(EO);
        }
        else if (producto=="IRS")
        {
            getIRS(deals.at(i).toInt(), &nominal, &settlement, &maturity, &divisa, &rol, &spread, &isFwd, &fwdStart,
                        &frecuenciaFija, &convencionFestivosFija, &convencionCalendarioFija, &tipoFijo, &curvaDescuento,
                        &frecuenciaFlotante,&convencionFestivosFlotante, &convencionCalendarioFlotante, &indice, &curvaForward);

            IRS=QString("%1;%2;%3;%4;%5;%6;%7;%8;%9;%10;%11;%12;%13;%14;%15;%16;%17;%18;%19").arg(deals.at(i)).arg(nominal).arg(settlement.toString()).arg(maturity.toString())
                    .arg(divisa).arg(rol).arg(spread).arg(isFwd).arg(fwdStart.toString()).arg(frecuenciaFija).arg(convencionFestivosFija)
                    .arg(convencionCalendarioFija).arg(tipoFijo).arg(curvaDescuento).arg(frecuenciaFlotante).arg(convencionFestivosFlotante).arg(convencionCalendarioFlotante)
                    .arg(indice).arg(curvaForward);
            listaIRS.append(IRS);
        }
    }
    QFile file_for_writing(fileName);
    file_for_writing.open(QIODevice::WriteOnly | QIODevice::Truncate);

    QTextStream text_stream_for_writing(&file_for_writing);

    //ESCRIBIMOS PRIMERO OPCIONES EUROPEAS

    text = "DEAL ID;NOMINAL;SETTLEMENT;MATURITY;DIVISA;TIPO DE OPCION;TIPO DE SUBYACENTE;SUBYACENTE;SPOT;STRIKE;TIPO LIBRE;VOLATILIDAD;CONVENCION DE CALENDARIO\n";
    text_stream_for_writing << text;
    for (int i=0; i<listaEO.count(); i++)
    {
        text_stream_for_writing << listaEO.at(i);
        text_stream_for_writing << "\n";
    }

    //DESPUES IRS
    text_stream_for_writing << "\n";

    text = "DEAL ID;NOMINAL;SETTLEMENT;MATURITY;DIVISA;ROL;SPREAD;ISFWD;FWDSTART;FRECUENCIA FIJA;CONVENCION FESTIVOS FIJA;CONVENCION CALENDARIO FIJA; TIPO FIJO; CURVA DE DESCUENTO;FRECUENCIA FLOTANTE;CONVENCION FESTIVOS FLOTANTE;CONVENCION CALENDARIO FLOTANTE;INDICE;CURVA FORWARD\n";
    text_stream_for_writing << text;
    for (int i=0; i<listaIRS.count(); i++)
    {
        text_stream_for_writing << listaIRS.at(i);
        text_stream_for_writing << "\n";
    }

    file_for_writing.close();


}
//-----------------------------------------------------------------------------
DBHandler::~DBHandler()
{
    db.removeDatabase(db.databaseName());
}
