//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#ifndef DBTOOLS_H
#define DBTOOLS_H

#include <QtSql>
#include <QTableWidget>
#include <QTreeWidget>
#include <QDate>
#include <QCheckBox>
#include <QComboBox>
#include <QDoubleSpinBox>
#include <QDateEdit>
#include <ql/quantlib.hpp>

#define TIPO_PRODUCTO 1000

class DBHandler
{


public:
    explicit DBHandler();
    bool conectar();
    int getDealsCount();
    bool getDeals(QStringList *lista);
    QSqlQuery *getColumnsFromTable(QString tableName);
    QStringList *getColumnsListFromTable(QString tableName);
    QSqlQuery *getDataFromTable(QString tableName);
    bool getDataFromTable(QTableWidget *tabla, QString tableName);
    bool getDataFromTable(QComboBox *combo, QString tableName);
    bool getDataFromTiposCambio(QTableWidget *tabla);
    bool getTiposFamilia(QComboBox *combo, QString tableName);
    bool updateCartera(QTreeWidget *tree);
    bool getFijacionesFromIndice(QTableWidget *tabla,QString indice);
    QSqlQuery *getDataFromDepositos();
    QSqlQuery *getDataFromFuturos();
    QSqlQuery *getDataFromFras();
    QSqlQuery *getDataFromSwaps();
    QString *getCalendarFromCurrency(QString divisa);
    bool getIntFieldFromTableforID(QString table, QString primary, QString id, QString field, int *ret);
    bool getDoubleFieldFromTableforID(QString table, QString primary, QString id, QString field, double *ret);
    bool getStringFieldFromTableforID(QString table, QString primary, QString id, QString field, QString *ret);
    bool getDataFromCurve(QTableWidget *tabla, QString curveID);
    bool getDataFromProductType(QComboBox *combo, QString tipo);
    bool getDataFromProductType(QTableWidget *tabla,QString tipo);
    double getTipoCambio(QString domestica, QString foranea);
    bool insertHolidayIntoCalendario(QString calendar, QDate date);
    bool insertIntoDivisas(QString iso, QString nombre, QString indice, int spot, QString calendario);
    bool insertIntoDepositos(QString id, QString currency, int plazo, QString periodo, double quote);
    bool insertIntoFuturos(QString id, QString currency, int meses, QString immdate, double quote);
    bool insertIntoFras(QString id, QString currency, int mesesStart, int mesesEnd, double quote);
    bool insertIntoSwaps(QString id, QString currency, int plazo, QString periodo, QString frecuenciaFija, QString convFest, QString convCal, QString indice, double quote);
    bool insertIntoCurvas(QString curvaID, QString divisa, double tolerancia, QString convencionCal, QStringList productos);
    bool insertIntoIndices(QString indiceID, QString indiceEstandar, int plazo, QString periodo);
    bool insertIntoDeals(QString product, double nominal, QDate settlementDate, QDate maturityDate, QString currency, QString type, double spread, int *id);
    bool insertIntoIrs(int isFwdStart, QDate fwdStart, int dealID);
    bool insertIntoEuropeanOption(QString tipoSubyacente, QString subyacente, double strike, double tipoLibre, double volatilidad, QString convencionCalendario, int dealID);
    bool insertIntoLeg( QString frequency, QString businessConv, QString calConv, QString curveName, QString legType, int *id);
    bool insertIntoFixedLeg(double fixedRate, int legID);
    bool insertIntoFloatingLeg(QString indexName, int legID);
    bool insertIntoFijaciones(QString indice, QDate fechaFijacion, double value);
    bool insertIntoTiposCambio(QString domestica, QString foranea, double quote);
    bool insertIRS(double nominal, QDate settlementDate, QDate maturityDate, QString currency, QString rol, double spread,
                   bool isForward, QDate fwdStart, QString fixedFrequency, QString fixedBusinessConv,
                   QString fixedCalConv, double fixedRate, QString discountCurveName, QString floatingFrequency,
                   QString floatingBusinessConv, QString floatingCalConv, QString indexName, QString forwardCurveName, int *id);
    bool insertEuropeanOption(  double nominal, QDate settlementDate, QDate maturityDate,
                                QString currency, QString tipoOpcion, QString tipoSubyacente, QString subyacente,
                                double strike, double tipoLibre, double volatilidad, QString convencionCalendario, int *id);
    QString *getFamilyFromProduct(QString product);
    QSqlQuery *getFamilies();
    QSqlQuery *getFamilyProducts(QString family);
    QSqlQuery *getDeals(QString product);
    void getOrderedDeals(QTableWidget *tabla);
    void getDeals(QTableWidget *tabla, QString product);
    QSqlQuery *getProducto(QString tipoProducto, QString idProducto);
    unsigned int getCurrencySpotSchedule(QString currency);
    QStringList *getCurvesContainingDivisa(QString divisa);
    QStringList *getDealsContainingDivisa(QString divisa);
    QStringList *getProductsContainingDivisa(QString divisa);
    QStringList *getCurvesContainingProduct(QString producto);
    QStringList *getDealsContainingCurve(QString curve);
    QStringList *getDealsContainingIndex(QString indice);
    bool updateQuote(QString producto, double newQuote);
    bool updateTipoCambio(QString domestic, QString foreign, double tipo);
    bool updateDefaultCurrency(QString defaultCurrency);
    bool getDefaultCurrency(QString *defaultCurrency);
    bool deleteHolidayFromCalendario(QString calendar, QDate date);
    bool deleteDivisa(QString divisa);
    bool deleteProducto(QString producto);
    bool deleteCurva(QString curva);
    bool deleteIndice(QString indice);
    bool deleteDeal(int dealID);
    bool deleteFijacion(QString indice, QString fechaFijacion);
    bool deleteTipoCambio(QString domestica, QString foranea);
    void addIndexFixings(QString indexName, boost::shared_ptr<QuantLib::IborIndex> index);
    void commit();
    void rollback();
    bool getIRS(int dealID, double *nominal, QDate *settlement, QDate *maturity,
                    QString *divisa, QString *rol, double *spread,
                    bool *isFwd, QDate *fwdStart,
                    QString *frecuenciaFija, QString *convencionFestivosFija,
                    QString *convencionCalendarioFija, double *tipoFijo,
                    QString *curvaDescuento, QString *frecuenciaFlotante,
                    QString *convencionFestivosFlotante, QString *convencionCalendarioFlotante,
                    QString *indice, QString *curvaForward);
    bool getIRS(int dealID, QDoubleSpinBox *doubleSpinBoxNominal, QDateEdit *dateEditSettlement, QDateEdit *dateEditMaturity,
                QComboBox *comboBoxDivisa, QComboBox *comboBoxRol, QDoubleSpinBox *doubleSpinBoxSpread,
                QCheckBox *checkBoxIsFwd, QDateEdit *dateEditFwdStart,
                QComboBox *comboBoxFrecuenciaFija, QComboBox *comboBoxConvencionFestivosFija,
                QComboBox *comboBoxConvencionCalendarioFija, QDoubleSpinBox *doubleSpinBoxTipoFijo,
                QComboBox *comboBoxCurvaDescuento, QComboBox *comboBoxFrecuenciaFlotante,
                QComboBox *comboBoxConvencionFestivosFlotante, QComboBox *comboBoxConvencionCalendarioFlotante,
                QComboBox *comboBoxIndice, QComboBox *comboBoxCurvaForward);
    bool getEuropeanOption( int dealID, QDoubleSpinBox *doubleSpinBoxNominal, QDateEdit *dateEditSettlement,
                            QDateEdit *dateEditMaturity, QComboBox *comboBoxDivisa, QComboBox *comboBoxTipoOpcion, QComboBox *comboBoxTipoSubyacente,
                            QComboBox *comboBoxSubyacente, QDoubleSpinBox *doubleSpinBoxSpot, QDoubleSpinBox *doubleSpinBoxStrike,
                            QDoubleSpinBox *doubleSpinBoxTipoLibre, QDoubleSpinBox *doubleSpinBoxVolatilidad, QComboBox *comboBoxConvencionCalendario);
    bool getEuropeanOption( int dealID, double *nominal, QDate *settlement,
                QDate *maturity, QString *divisa, QString *tipoOpcion, QString *tipoSubyacente, QString *subyacente,
                double *spot, double *strike,double *tipoLibre, double *volatilidad, QString *convencionCalendario);
    void exportDealsToFile(QString fileName);
    ~DBHandler();

private:
    QSqlDatabase db;
};

#endif // DBTOOLS_H
