//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "calendariowindow.h"
#include "ui_calendariowindow.h"
#include <QMessageBox>
#include <DBTools.h>
#include <valoracion.h>
#include <vector>

CalendarioWindow::CalendarioWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CalendarioWindow)
{
    using namespace QuantLib;
    extern DBHandler *DB;

    ui->setupUi(this);

    DB->getDataFromTable(ui->comboBoxCalendario,"calendarios");
    refreshTable();
}

CalendarioWindow::~CalendarioWindow()
{
    delete ui;
}

void CalendarioWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}
void CalendarioWindow::refreshTable(QString calendario)
{

    int numElements=0;
    QString titulos("FECHA");
    QStringList headers=titulos.split(";");


    int day, month, year;

    using namespace QuantLib;

    Calendar *cal;
    QString calName;

    if (calendario!="")
        calName=calendario;
    else
        calName=ui->comboBoxCalendario->currentText();

    cal=getCalendar(calName);
    adjustCalendarHolidays(calName, cal);
    Date d1 (24 , Dec ,2000);
    Date d2 (30 , Dec ,2099);
    std::vector < Date > holidays = cal->holidayList( *cal, d1 ,d2 , false );
    numElements=holidays.size();
    ui->tableWidgetFestivos->setRowCount(numElements);
    ui->tableWidgetFestivos->setColumnCount(1);
    ui->tableWidgetFestivos->setHorizontalHeaderLabels(headers);
    for (int i=0; i<numElements; i++)
    {
          day=holidays[i].dayOfMonth();
          month=holidays[i].month();
          year=holidays[i].year();
          QString dateString = QString(tr("%1/%2/%3")).arg(day).arg(month).arg(year);
          ui->tableWidgetFestivos->setItem(i, 0, new QTableWidgetItem(dateString));
    }
}


void CalendarioWindow::on_pushButtonEliminar_clicked()
{
    extern DBHandler *DB;
    bool bret=false;
    bret=DB->deleteHolidayFromCalendario(ui->comboBoxCalendario->currentText(), ui->dateEditFestivo->date());
    if (bret)
    {
        QMessageBox::information(0,"Eliminar","Festivo eliminado con exito");
        refreshTable();
    }
}

void CalendarioWindow::on_pushButtonAnadir_clicked()
{
    extern DBHandler *DB;
    bool bret=false;
    bret=DB->insertHolidayIntoCalendario(ui->comboBoxCalendario->currentText(), ui->dateEditFestivo->date());
    if (bret)
    {
        QMessageBox::information(0,"Insercion","Festivo incluido con exito");
        refreshTable();
    }
}

void CalendarioWindow::on_calendarWidget_clicked(const QDate &date)
{
    ui->dateEditFestivo->setDate(date);
}

void CalendarioWindow::on_tableWidgetFestivos_cellClicked(int row, int column)
{
    ui->tableWidgetFestivos->selectRow(row);
}

void CalendarioWindow::on_comboBoxCalendario_currentIndexChanged(const QString &arg1)
{
    refreshTable(arg1);
}
