//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "dealswindow.h"
#include "ui_dealswindow.h"
#include "DBTools.h"
#include <QMessageBox>
#include <QKeyEvent>

DealsWindow::DealsWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DealsWindow)
{
    extern DBHandler *DB;
    ui->setupUi(this);
    DB->getDataFromTable(ui->comboBoxFamilia,"familias");
    DB->getDataFromTable(ui->comboBoxFamiliaIns,"familias");
    DB->getTiposFamilia(ui->comboBoxTipo,ui->comboBoxFamilia->currentText());
    DB->getTiposFamilia(ui->comboBoxTipoIns,ui->comboBoxFamiliaIns->currentText());
    ui->labelFamilia->hide();
    ui->labelTipo->hide();
    ui->comboBoxFamilia->hide();
    ui->comboBoxTipo->hide();
    DB->getDataFromTable(ui->tablaDeals,"deals");
}

DealsWindow::~DealsWindow()
{
    delete ui;
}

void DealsWindow::on_comboBoxFamilia_currentIndexChanged(const QString &arg1)
{
    extern DBHandler *DB;
    DB->getTiposFamilia(ui->comboBoxTipo,arg1);
    DB->getDeals(ui->tablaDeals,ui->comboBoxTipo->currentText());

}

void DealsWindow::on_comboBoxFamiliaIns_currentIndexChanged(const QString &arg1)
{
    extern DBHandler *DB;
    DB->getTiposFamilia(ui->comboBoxTipoIns,arg1);
}

void DealsWindow::on_checkBoxFiltrar_clicked(bool checked)
{
    extern DBHandler *DB;
    if (checked)
    {
        ui->labelFamilia->show();
        ui->labelTipo->show();
        ui->comboBoxFamilia->show();
        ui->comboBoxTipo->show();
        DB->getDeals(ui->tablaDeals,ui->comboBoxTipo->currentText());
    }
    else
    {
        ui->labelFamilia->hide();
        ui->labelTipo->hide();
        ui->comboBoxFamilia->hide();
        ui->comboBoxTipo->hide();
        DB->getDataFromTable(ui->tablaDeals,"deals");
    }
}

void DealsWindow::on_pushButtonInsertar_clicked()
{
    int iRet=-1;
    extern DBHandler *DB;
    QString producto=ui->comboBoxTipoIns->currentText();
    if (producto=="IRS")
    {
        IRSWindow *ventanaIRS = new IRSWindow(0);
        iRet=ventanaIRS->exec();
        if( iRet == QDialog::Accepted )
        {
            DB->getDataFromTable(ui->tablaDeals,"deals");
        }
    }
    else if (producto=="European Option")
    {
        EuropeanOptionWindow *ventanaEuropeanOption= new EuropeanOptionWindow(0);
        iRet=ventanaEuropeanOption->exec();
        if( iRet == QDialog::Accepted )
        {
            DB->getDataFromTable(ui->tablaDeals,"deals");
        }
    }
    else
        QMessageBox::warning(0,"Producto","Producto no disponible");
}

void DealsWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}

void DealsWindow::on_tablaDeals_cellClicked(int row, int column)
{
    ui->tablaDeals->selectRow(row);
}

void DealsWindow::on_pushButtonEliminar_clicked()
{
    extern DBHandler *DB;
    bool bret=false;
    int dealID=-1;
    if (!ui->tablaDeals->rowCount())
    {
        QMessageBox::warning(0,"Eliminar","La tabla esta vacia");
        return;
    }
    int currentRow=ui->tablaDeals->currentRow();
    dealID=ui->tablaDeals->item(currentRow,0)->text().toInt();
    bret=DB->deleteDeal(dealID);
    if (bret)
        QMessageBox::information(0,"Eliminar","Producto eliminado con exito");
    else
        QMessageBox::critical(0,"Eliminar","No se pudo eliminar el producto");
    DB->getDataFromTable(ui->tablaDeals,"deals");
}

void DealsWindow::on_pushButtonEditarSeleccionado_clicked()
{
    int iRet=-1;
    extern DBHandler *DB;
    if (!ui->tablaDeals->rowCount())
    {
        QMessageBox::warning(0,"Editar","La tabla esta vacia");
        return;
    }
    int id=ui->tablaDeals->item(ui->tablaDeals->currentRow(),0)->text().toInt();
    QString producto=ui->tablaDeals->item(ui->tablaDeals->currentRow(),1)->text();
    if (producto=="IRS")
    {
        IRSWindow *ventanaIRS = new IRSWindow(id);
        iRet=ventanaIRS->exec();
        if( iRet == QDialog::Accepted )
        {
            DB->getDataFromTable(ui->tablaDeals,"deals");
        }
    }
    else if (producto=="European Option")
    {
        EuropeanOptionWindow *ventanaEuropeanOption = new EuropeanOptionWindow(id);
        iRet=ventanaEuropeanOption->exec();
        if( iRet == QDialog::Accepted )
        {
            DB->getDataFromTable(ui->tablaDeals,"deals");
        }
        delete ventanaEuropeanOption;
    }
    else
        QMessageBox::warning(0,"Producto","Producto no disponible");


}

void DealsWindow::on_pushButton_clicked()
{
    int iRet=-1;
    extern DBHandler *DB;
    QString producto;
    bool bret=false;
    QString id=QString("%1").arg(ui->spinBoxDeal->value());
    bret=DB->getStringFieldFromTableforID("deals", "ID", id, "PRODUCT", &producto);
    if (!bret)
    {
        QMessageBox::warning(0,"Seleccion","Producto no encontrado");
        return;
    }
    if (producto=="IRS")
    {
        IRSWindow *ventanaIRS = new IRSWindow(id.toInt());
        iRet=ventanaIRS->exec();
        if( iRet == QDialog::Accepted )
        {
            DB->getDataFromTable(ui->tablaDeals,"deals");
        }
    }
    else if (producto=="European Option")
    {
        EuropeanOptionWindow *ventanaEuropeanOption = new EuropeanOptionWindow(id.toInt());
        iRet=ventanaEuropeanOption->exec();
        if( iRet == QDialog::Accepted )
        {
            DB->getDataFromTable(ui->tablaDeals,"deals");
        }
    }

    else
       QMessageBox::warning(0,"Seleccion","Producto no disponible");
}

void DealsWindow::on_comboBoxTipo_currentIndexChanged(const QString &arg1)
{
    extern DBHandler *DB;
    DB->getDeals(ui->tablaDeals,arg1);
}
