//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "depositoswindow.h"
#include "ui_depositoswindow.h"
#include "curvaswindow.h"
#include "DBTools.h"
#include <QMessageBox>
#include <QValidator>
#include <QIntValidator>
#include <QDoubleValidator>
#include <QKeyEvent>

DepositosWindow::DepositosWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DepositosWindow)
{
    ui->setupUi(this);
    extern DBHandler *DB;
    int fieldno=-1;

    initializing=true;   //para no activar los triggers sobre celdas de la tabla

    QValidator *val=new QIntValidator(1,1000,ui->lineEditPlazo);
    ui->lineEditPlazo->setValidator(val);
    QValidator *val1=new QDoubleValidator(0,999999,8,ui->lineEditMQ);
    ui->lineEditMQ->setValidator(val1);



    DB->getDataFromProductType(ui->tableWidget, "Deposito");

    QSqlQuery *queryDivisas=DB->getDataFromTable("divisas");
    while (queryDivisas->next())
    {
        fieldno = queryDivisas->record().indexOf("CURRENCY_ISO");
        QString txt(queryDivisas->value(fieldno).toString());
        ui->comboBoxDivisa->addItem(txt);
    }
    if (queryDivisas)
        delete queryDivisas;

    ui->comboBoxPeriodo->addItem("Dia");
    ui->comboBoxPeriodo->addItem("Semana");
    ui->comboBoxPeriodo->addItem("Mes");
    ui->comboBoxPeriodo->addItem("Anio");

    initializing=false;
}

DepositosWindow::~DepositosWindow()
{
    delete ui;
}

void DepositosWindow::on_addDepoButton_clicked()
{
    extern DBHandler *DB;
    bool result=false;
    if ((ui->lineEditID->text()=="") || (ui->lineEditMQ->text()=="") || (ui->lineEditPlazo->text()==""))
    {
        QMessageBox::warning(0,"Insercion","Es necesario completar todos los campos");
        return;
    }

    result=DB->insertIntoDepositos(ui->lineEditID->text(), ui->comboBoxDivisa->currentText(), ui->lineEditPlazo->text().toInt(), ui->comboBoxPeriodo->currentText(), ui->lineEditMQ->text().toDouble());

    if (result)
        DB->getDataFromProductType(ui->tableWidget, "Deposito");
}

void DepositosWindow::on_delDepoButton_clicked()
{
    extern DBHandler *DB;
    int numCurvas=0;
    if (!ui->tableWidget->rowCount())
    {
        QMessageBox::warning(0,"Eliminar","La tabla esta vacia");
        return;
    }
    QString mensaje = "Se va a proceder a borrar el producto: ";
    QStringList *lista;
    int row=ui->tableWidget->currentRow();
    QString producto=ui->tableWidget->item(row,0)->text();
    mensaje.append(producto);

    switch( QMessageBox::warning( this, "Finanval",mensaje,"Si","No", 0, 0, 1 ))
    {
    case 0:
        lista=DB->getCurvesContainingProduct(producto);
        numCurvas=lista->count();
        if (numCurvas)
        {
            mensaje="El producto esta incluido en las curvas: ";
            for (int i=0; i<numCurvas; i++)
            {
                if (i)
                    mensaje.append(", ");
                mensaje.append(lista->value(i));
            }
            mensaje.append("\nElimine o sustituya este producto en las curvas antes de proceder");
            switch (QMessageBox::critical( this, "Finanval",mensaje,"Ok", "Ver curvas", 0, 0, 1 ))
            {
            case 0:
                delete lista;
                return;

            case 1:
                CurvasWindow *ventanaCurvas = new CurvasWindow();
                ventanaCurvas->exec();
            }
        }
        else
        {
            if (DB->deleteProducto(producto)) //Si todo fue bien repintamos la tabla
                DB->getDataFromProductType(ui->tableWidget, "Deposito");
        }
        delete lista;
        break;
    case 1:
        return;
        break;
    }

}

void DepositosWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}

void DepositosWindow::on_tableWidget_cellChanged(int row, int column)
{
    extern DBHandler *DB;

    if ((!initializing) && (column==ui->tableWidget->columnCount()-1))
    {
        QString producto(ui->tableWidget->item(row,0)->text());
        DB->updateQuote(producto,ui->tableWidget->item(row,column)->text().toDouble());
    }
}

void DepositosWindow::on_cleanButton_clicked()
{
    ui->lineEditID->setText("");
    ui->lineEditMQ->setText("");
    ui->lineEditPlazo->setText("");
}

void DepositosWindow::on_refrescarButton_clicked()
{
    extern DBHandler *DB;
    DB->getDataFromProductType(ui->tableWidget, "Deposito");
}

