//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "divisaswindow.h"
#include "ui_divisaswindow.h"
#include "curvaswindow.h"
#include "DBTools.h"
#include <QMessageBox>
#include <QValidator>
#include <QIntValidator>
#include <QDoubleValidator>
#include <QKeyEvent>

DivisasWindow::DivisasWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DivisasWindow)
{
    extern DBHandler *DB;
    ui->setupUi(this);
    ui->lineEditISO->setMaxLength(3);
    DB->getDataFromTable(ui->tableWidgetDivisas,"divisas");
    DB->getDataFromTable(ui->comboBoxCalendario,"calendarios");
    DB->getDataFromTable(ui->comboBoxIndice,"indices_estandar");

}

DivisasWindow::~DivisasWindow()
{
    delete ui;
}

void DivisasWindow::on_addDivisaButton_clicked()
{
    extern DBHandler *DB;
    bool result=false;

    if ((ui->lineEditISO->text()=="") || (ui->lineEditNombre->text()==""))
    {
        QMessageBox::warning(0,"Insercion","Es necesario completar todos los campos");
        return;
    }
    result=DB->insertIntoDivisas(ui->lineEditISO->text(), ui->lineEditNombre->text(), ui->comboBoxIndice->currentText(), ui->spinBox->value(), ui->comboBoxCalendario->currentText());
    if (result)
    {
        QMessageBox::information(0,"Insercion en Base de Datos","Divisa insertada con exito");
        DB->getDataFromTable(ui->tableWidgetDivisas,"divisas");
    }
}


void DivisasWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}


void DivisasWindow::on_cleanButton_clicked()
{
    ui->lineEditISO->setText("");
    ui->lineEditNombre->setText("");
    ui->spinBox->setValue(0);
}


void DivisasWindow::on_delDivisaButton_clicked()
{
    extern DBHandler *DB;
    int numCurvas=0;
    if (!ui->tableWidgetDivisas->rowCount())
    {
        QMessageBox::warning(0,"Eliminar","La tabla esta vacia");
        return;
    }
    QString mensaje = "Se va a proceder a borrar la divisa: ";
    QStringList *lista;
    int row=ui->tableWidgetDivisas->currentRow();
    QString divisa=ui->tableWidgetDivisas->item(row,0)->text();
    mensaje.append(divisa);

    switch( QMessageBox::warning( this, "Finanval",mensaje,"Si","No", 0, 0, 1 ))
    {
    case 0:
        lista=DB->getCurvesContainingDivisa(divisa);
        numCurvas=lista->count();
        if (numCurvas)
        {
            mensaje="El producto esta incluido en las curvas: ";
            for (int i=0; i<numCurvas; i++)
            {
                if (i)
                    mensaje.append(", ");
                mensaje.append(lista->value(i));
            }
            mensaje.append("\nElimine o sustituya este producto en las curvas antes de proceder");
            switch (QMessageBox::critical( this, "Finanval",mensaje,"Ok", "Ver curvas", 0, 0, 1 ))
            {
            case 0:
                delete lista;
                return;

            case 1:
                CurvasWindow *ventanaCurvas = new CurvasWindow();
                ventanaCurvas->exec();
                delete lista;
            }
        }
        else
        {
            lista=DB->getDealsContainingDivisa(divisa);
            int numDeals=lista->count();
            if (numDeals)
            {
                mensaje="La divisa esta incluida en las contratos: ";
                for (int i=0; i<numDeals; i++)
                {
                    if (i)
                        mensaje.append(", ");
                    mensaje.append(lista->value(i));
                }
                mensaje.append("\nSustituya esta divisa en los contratos antes de proceder");
                delete lista;
                return;

            }
            else
            {
                lista=DB->getProductsContainingDivisa(divisa);
                int numProductos=lista->count();
                if (numProductos)
                {
                    mensaje="La divisa esta incluida en los productos: ";
                    for (int i=0; i<numDeals; i++)
                    {
                        if (i)
                            mensaje.append(", ");
                        mensaje.append(lista->value(i));
                    }
                    mensaje.append("\nSustituya esta divisa en los productos antes de proceder");
                    delete lista;
                    return;
                }

            }
            if (DB->deleteDivisa(divisa)) //Si todo fue bien repintamos la tabla
                DB->getDataFromTable(ui->tableWidgetDivisas,"divisas");
        }
        break;
    case 1:
        return;
        break;
    }

}

void DivisasWindow::on_tableWidgetDivisas_cellClicked(int row, int column)
{
    ui->tableWidgetDivisas->selectRow(row);
}
