//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "europeanoptionwindow.h"
#include "ui_europeanoptionwindow.h"
#include "DBTools.h"
#include "valoracion.h"
#include <QPalette>
#include <QKeyEvent>
#include <ql/quantlib.hpp>
#include <QMessageBox>
#include <QPalette>
#include <flujoswindow.h>
#include <QTableWidget>
#include <iostream>

EuropeanOptionWindow::EuropeanOptionWindow(int ID, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::EuropeanOptionWindow)
{
    extern DBHandler *DB;
    ui->setupUi(this);
    dealID=ID;


    //Setting labels
    QString *familia=DB->getFamilyFromProduct("EuropeanOption");
    ui->labelFamilia->setText(*familia);
    ui->labelFamilia->setStyleSheet("QLabel { color : blue; }");
    ui->labelProducto->setText("EuropeanOption");
    ui->labelProducto->setStyleSheet("QLabel { color : blue; }");
    if (!dealID)
        ui->labelIDContrato->setText("Nuevo");
    else
    {
        QString mID=QString("%1").arg(dealID);
        ui->labelIDContrato->setText(mID);
    }
    ui->labelIDContrato->setStyleSheet("QLabel { color : red; }");

    //Setting comboBox
    DB->getDataFromTable(ui->comboBoxConvencionCalendario,"convenciones_calendario");
    DB->getDataFromTable(ui->comboBoxDivisa,"divisas");
    DB->getDataFromTable(ui->comboBoxTipoSubyacente,"tipo_producto_mercado");
    DB->getDataFromProductType(ui->comboBoxSubyacente, ui->comboBoxTipoSubyacente->currentText());
    ui->comboBoxTipoOpcion->addItem("Put");
    ui->comboBoxTipoOpcion->addItem("Call");

    //Setting double input
    ui->doubleSpinBoxNominal->setRange(0.,999999999999.99);
    ui->doubleSpinBoxNominal->setDecimals(8);
    ui->doubleSpinBoxSpot->setRange(0.,99999999999.9999999999);
    ui->doubleSpinBoxSpot->setDecimals(8);
    ui->doubleSpinBoxStrike->setRange(0.,99999999999.9999999999);
    ui->doubleSpinBoxStrike->setDecimals(8);
    ui->doubleSpinBoxTipoLibre->setRange(0.,999.9999999999);
    ui->doubleSpinBoxTipoLibre->setDecimals(8);
    ui->doubleSpinBoxVolatilidad->setRange(0.,100.0000000000);
    ui->doubleSpinBoxVolatilidad->setDecimals(8);

    if (dealID)
    {
        DB->getEuropeanOption( dealID, ui->doubleSpinBoxNominal, ui->dateEditSettlement, ui->dateEditMaturity,
                    ui->comboBoxDivisa, ui->comboBoxTipoOpcion, ui->comboBoxTipoSubyacente, ui->comboBoxSubyacente, ui->doubleSpinBoxSpot,
                    ui->doubleSpinBoxStrike, ui->doubleSpinBoxTipoLibre, ui->doubleSpinBoxVolatilidad,
                    ui->comboBoxConvencionCalendario);
    }

    //Settings results

    ui->lineEditNPV->setReadOnly(true);
    ui->lineEditElasticidad->setReadOnly(true);
}

EuropeanOptionWindow::~EuropeanOptionWindow()
{
    delete ui;
}

void EuropeanOptionWindow::on_pushButtonDesdeCalSettlement_clicked()
{
    ui->dateEditSettlement->setDate(ui->calendarWidget->selectedDate());
}

void EuropeanOptionWindow::on_pushButtonDesdeCalMaturity_clicked()
{
        ui->dateEditMaturity->setDate(ui->calendarWidget->selectedDate());
}

void EuropeanOptionWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}

void EuropeanOptionWindow::on_pushButtonValorar_clicked()
{

    QuantLib::VanillaOption *europeanOption=NULL;
    double NPV=0.0;
    double elasticity=0.0;
    try {
        europeanOption=getEuropeanOption( ui->doubleSpinBoxNominal->value(), ui->dateEditSettlement->date(), ui->dateEditMaturity->date(),
                                          ui->comboBoxDivisa->currentText(), ui->comboBoxTipoOpcion->currentText(), ui->doubleSpinBoxSpot->value(),
                                          ui->doubleSpinBoxStrike->value(), ui->doubleSpinBoxTipoLibre->value(),
                                          ui->doubleSpinBoxVolatilidad->value(), ui->comboBoxConvencionCalendario->currentText());
        if (!europeanOption)
            return;
        NPV = europeanOption->NPV();
        NPV*=ui->doubleSpinBoxNominal->value();
        elasticity = europeanOption->elasticity();
    }
    catch ( std :: exception & e)
    {
        QMessageBox::critical(0,"Error en valoracion",e.what());
        delete europeanOption;
        return ;
    } catch (...)
    {
        QMessageBox::critical(0,"Error en valoracion","Error desconocido");
        delete europeanOption;
        return ;
    }

    QPalette palNPV;
    QPalette palElasticity;

    if (NPV<0)
        palNPV.setColor(QPalette::Text, Qt::red);
    else
        palNPV.setColor(QPalette::Text, Qt::blue);

    if (elasticity<0)
        palElasticity.setColor(QPalette::Text, Qt::red);
    else
        palElasticity.setColor(QPalette::Text, Qt::blue);


    ui->lineEditNPV->setPalette(palNPV);
    ui->lineEditElasticidad->setPalette(palElasticity);

    ui->lineEditNPV->setText(QString("%1").arg(NPV));
    ui->lineEditElasticidad->setText(QString("%1").arg(elasticity));
    delete europeanOption;
}

void EuropeanOptionWindow::on_buttonBox_clicked(QAbstractButton *button)
{
    extern DBHandler *DB;
    bool deleted=false, inserted=false;
    if (button==ui->buttonBox->button(QDialogButtonBox::Save))
    {

        if (dealID)
        {
            deleted=DB->deleteDeal(dealID);
            if (!deleted)
            {
                QMessageBox::critical(0,"Guardar","No se pudo guardar el EuropeanOption");
                return;
            }

        }
        inserted=DB->insertEuropeanOption(ui->doubleSpinBoxNominal->value(), ui->dateEditSettlement->date(), ui->dateEditMaturity->date(),
                ui->comboBoxDivisa->currentText(), ui->comboBoxTipoOpcion->currentText(), ui->comboBoxTipoSubyacente->currentText(),
                ui->comboBoxSubyacente->currentText(), ui->doubleSpinBoxStrike->value(), ui->doubleSpinBoxTipoLibre->value(),
                ui->doubleSpinBoxVolatilidad->value(), ui->comboBoxConvencionCalendario->currentText(), &dealID);
        if (inserted)
        {
            QString msg= QString("Contrato guardado con exito, identificador %1").arg(dealID);
            QMessageBox::information(0,"Guardar",msg);
        }
        else
            QMessageBox::critical(0,"Guardar","No se pudo guardar el EuropeanOption");
    }
    else
        this->close();
}

void EuropeanOptionWindow::on_comboBoxSubyacente_currentIndexChanged(const QString &arg1)
{
    extern DBHandler *DB;
    double quote;
    if (arg1!="")
    {
        DB->getDoubleFieldFromTableforID("productos", "ID", arg1, "QUOTE", &quote);
        if (!quote)
            ui->doubleSpinBoxSpot->setValue(0.0);
        else
            ui->doubleSpinBoxSpot->setValue(quote);
    }
}

void EuropeanOptionWindow::on_comboBoxTipoSubyacente_currentIndexChanged(const QString &arg1)
{
    extern DBHandler *DB;
    DB->getDataFromProductType(ui->comboBoxSubyacente, arg1);
}
