//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "fraswindow.h"
#include "ui_fraswindow.h"
#include "curvaswindow.h"
#include "DBTools.h"
#include <QMessageBox>
#include <QValidator>
#include <QIntValidator>
#include <QDoubleValidator>
#include <QKeyEvent>

FrasWindow::FrasWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::FrasWindow)
{
    ui->setupUi(this);
    extern DBHandler *DB;
    int fieldno=-1;

    initializing=true;   //para no activar los triggers sobre celdas de la tabla

    QValidator *val=new QIntValidator(1,1000,ui->lineEditStart);
    ui->lineEditStart->setValidator(val);
    QValidator *val1=new QIntValidator(1,1000,ui->lineEditEnd);
    ui->lineEditEnd->setValidator(val1);
    QValidator *val2=new QDoubleValidator(0,999999,8,ui->lineEditMQ);
    ui->lineEditMQ->setValidator(val2);



    DB->getDataFromProductType(ui->tableWidget, "Fra");

    QSqlQuery *queryDivisas=DB->getDataFromTable("divisas");
    while (queryDivisas->next())
    {
        fieldno = queryDivisas->record().indexOf("CURRENCY_ISO");
        QString txt(queryDivisas->value(fieldno).toString());
        ui->comboBoxDivisa->addItem(txt);
    }
    if (queryDivisas)
        delete queryDivisas;

    initializing=false;
}

FrasWindow::~FrasWindow()
{
    delete ui;
}

void FrasWindow::on_addFraButton_clicked()
{
    extern DBHandler *DB;
    bool result=false;
    if ((ui->lineEditID->text()=="") || (ui->lineEditStart->text()=="") || (ui->lineEditEnd->text()=="") ||(ui->lineEditMQ->text()==""))
    {
        QMessageBox::warning(0,"Insercion","Es necesario completar todos los campos");
        return;
    }
    if ((ui->lineEditStart->text().toInt()) >= (ui->lineEditEnd->text().toInt()))
    {
        QMessageBox::warning(0,"Insercion","Meses a Inicio debe ser inferior a Meses a Fin");
        return;
    }


    result=DB->insertIntoFras(ui->lineEditID->text(), ui->comboBoxDivisa->currentText(), ui->lineEditStart->text().toInt(), ui->lineEditEnd->text().toInt(), ui->lineEditMQ->text().toDouble());

    if (result)
        DB->getDataFromProductType(ui->tableWidget, "Fra");
}

void FrasWindow::on_delFraButton_clicked()
{
    extern DBHandler *DB;
    int numCurvas=0;
    QString mensaje = "Se va a proceder a borrar el producto: ";
    QStringList *lista;
    if (!ui->tableWidget->rowCount())
    {
        QMessageBox::warning(0,"Eliminar","La tabla esta vacia");
        return;
    }
    int row=ui->tableWidget->currentRow();
    QString producto=ui->tableWidget->item(row,0)->text();
    mensaje.append(producto);

    switch( QMessageBox::warning( this, "Finanval",mensaje,"Si","No", 0, 0, 1 ))
    {
    case 0:
        lista=DB->getCurvesContainingProduct(producto);
        numCurvas=lista->count();
        if (numCurvas)
        {
            mensaje="El producto esta incluido en las curvas: ";
            for (int i=0; i<numCurvas; i++)
            {
                if (i)
                    mensaje.append(", ");
                mensaje.append(lista->value(i));
            }
            mensaje.append("\nElimine o sustituya este producto en las curvas antes de proceder");
            switch (QMessageBox::critical( this, "Finanval",mensaje,"Ok", "Ver curvas", 0, 0, 1 ))
            {
            case 0:
                delete lista;
                return;

            case 1:
                CurvasWindow *ventanaCurvas = new CurvasWindow();
                ventanaCurvas->exec();
            }
        }
        else
        {
            if (DB->deleteProducto(producto)) //Si todo fue bien repintamos la tabla
                DB->getDataFromProductType(ui->tableWidget, "Fra");
        }
        delete lista;
        break;
    case 1:
        return;
        break;
    }

}

void FrasWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}

void FrasWindow::on_tableWidget_cellChanged(int row, int column)
{
    extern DBHandler *DB;
    if ((!initializing) && (column==ui->tableWidget->columnCount()-1))
    {
        QString producto(ui->tableWidget->item(row,0)->text());
        DB->updateQuote(producto,ui->tableWidget->item(row,column)->text().toDouble());
    }
}

void FrasWindow::on_cleanButton_clicked()
{
    ui->lineEditID->setText("");
    ui->lineEditMQ->setText("");
    ui->lineEditStart->setText("");
    ui->lineEditEnd->setText("");
}

void FrasWindow::on_refrescarButton_clicked()
{
    extern DBHandler *DB;
    DB->getDataFromProductType(ui->tableWidget, "Fra");
}
