//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "futuroswindow.h"
#include "ui_futuroswindow.h"
#include "DBTools.h"
#include <QMessageBox>
#include "curvaswindow.h"
#include <ql/quantlib.hpp>
#include <QKeyEvent>

FuturosWindow::FuturosWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::FuturosWindow)
{
    ui->setupUi(this);

    extern DBHandler *DB;
    int fieldno=-1;
    initializing=true;   //para no activar los triggers sobre celdas de la tabla

    QValidator *val=new QIntValidator(1,1000,ui->lineEditMeses);
    ui->lineEditMeses->setValidator(val);
    QValidator *val1=new QDoubleValidator(0,999999,8,ui->lineEditMQ);
    ui->lineEditMQ->setValidator(val1);


    DB->getDataFromProductType(ui->tableWidget, "Futuro");

    QSqlQuery *queryDivisas=DB->getDataFromTable("divisas");
    while (queryDivisas->next())
    {
        fieldno = queryDivisas->record().indexOf("CURRENCY_ISO");
        QString txt(queryDivisas->value(fieldno).toString());
        ui->comboBoxDivisa->addItem(txt);
    }
    if (queryDivisas)
        delete queryDivisas;
    initializing=false;

    QuantLib::Date today=QuantLib::Date::todaysDate();
    QuantLib::Date imm = QuantLib::IMM::nextDate(today);

    QString date;
    int day=imm.dayOfMonth();
    int month=imm.month();
    int year=imm.year();
    QString dateString = QString(tr("%1-%2-%3")).arg(year).arg(month).arg(day);
    ui->comboBoxIMMDate->addItem(dateString);
    imm= QuantLib::IMM::nextDate(imm+1);
    day=imm.dayOfMonth();
    month=imm.month();
    year=imm.year();
    QString dateString1 = QString(tr("%1-%2-%3")).arg(year).arg(month).arg(day);
    ui->comboBoxIMMDate->addItem(dateString1);
    imm= QuantLib::IMM::nextDate(imm+1);
    day=imm.dayOfMonth();
    month=imm.month();
    year=imm.year();
    QString dateString2 = QString(tr("%1-%2-%3")).arg(year).arg(month).arg(day);
    ui->comboBoxIMMDate->addItem(dateString2);
    imm= QuantLib::IMM::nextDate(imm+1);
    day=imm.dayOfMonth();
    month=imm.month();
    year=imm.year();
    QString dateString3 = QString(tr("%1-%2-%3")).arg(year).arg(month).arg(day);
    ui->comboBoxIMMDate->addItem(dateString3);

}

FuturosWindow::~FuturosWindow()
{
    delete ui;
}

void FuturosWindow::on_addFuturoButton_clicked()
{
    extern DBHandler *DB;
    bool result=false;

    if ((ui->lineEditID->text()=="") || (ui->lineEditMeses->text()=="") || (ui->lineEditMQ->text()==""))
    {
        QMessageBox::warning(0,"Insercion","Es necesario completar todos los campos");
        return;
    }
    result=DB->insertIntoFuturos(ui->lineEditID->text(), ui->comboBoxDivisa->currentText(), ui->lineEditMeses->text().toInt(),  ui->comboBoxIMMDate->currentText(), ui->lineEditMQ->text().toDouble());

    if (result)
        DB->getDataFromProductType(ui->tableWidget, "Futuro");
}

void FuturosWindow::on_cleanButton_clicked()
{
    ui->lineEditID->setText("");
    ui->lineEditMeses->setText("");
    ui->lineEditMQ->setText("");
}

void FuturosWindow::on_delFuturoButton_clicked()
{
    extern DBHandler *DB;
    int numCurvas=0;
    QString mensaje = "Se va a proceder a borrar el producto: ";
    QStringList *lista;
    if (!ui->tableWidget->rowCount())
    {
        QMessageBox::warning(0,"Eliminar","La tabla esta vacia");
        return;
    }
    int row=ui->tableWidget->currentRow();
    QString producto=ui->tableWidget->item(row,0)->text();
    mensaje.append(producto);

    switch( QMessageBox::warning( this, "Finanval",mensaje,"Si","No", 0, 0, 1 ))
    {
    case 0:
        lista=DB->getCurvesContainingProduct(producto);
        numCurvas=lista->count();
        if (numCurvas)
        {
            mensaje="El producto esta incluido en las curvas: ";
            for (int i=0; i<numCurvas; i++)
            {
                if (i)
                    mensaje.append(", ");
                mensaje.append(lista->value(i));
            }
            mensaje.append("\nElimine o sustituya este producto en las curvas antes de proceder");
            switch (QMessageBox::critical( this, "Finanval",mensaje,"Ok", "Ver curvas", 0, 0, 1 ))
            {
            case 0:
                delete lista;
                return;

            case 1:
                CurvasWindow *ventanaCurvas = new CurvasWindow();
                ventanaCurvas->exec();
            }
        }
        else
        {
            if (DB->deleteProducto(producto)) //Si todo fue bien repintamos la tabla
                DB->getDataFromProductType(ui->tableWidget, "Futuro");
        }
        delete lista;
        break;
    case 1:
        return;
        break;
    }
}

void FuturosWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}

void FuturosWindow::on_tableWidget_cellChanged(int row, int column)
{
    extern DBHandler *DB;
    bool ret=false;
    if ((!initializing) && (column==ui->tableWidget->columnCount()-1))
    {
        QString producto(ui->tableWidget->item(row,0)->text());
        ret=DB->updateQuote(producto,ui->tableWidget->item(row,column)->text().toDouble());
        if (!ret) //restauramos el valor original porque no se pudo insertar
            DB->getDataFromProductType(ui->tableWidget, "Deposito");
    }
}

void FuturosWindow::on_pushButton_clicked()
{
    extern DBHandler *DB;
    DB->getDataFromProductType(ui->tableWidget, "Futuro");
}
