//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "indiceswindow.h"
#include "ui_indiceswindow.h"
#include "DBTools.h"
#include <QMessageBox>
#include <QValidator>
#include <QIntValidator>
#include <QDoubleValidator>
#include <QKeyEvent>
#include <fijacioneswindow.h>

IndicesWindow::IndicesWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::IndicesWindow)
{
    ui->setupUi(this);
    extern DBHandler *DB;
    QString descripcion;

    QValidator *val=new QIntValidator(1,1000,ui->lineEditPlazo);
    ui->lineEditPlazo->setValidator(val);

    DB->getDataFromTable(ui->tableWidget,"indices");
    DB->getDataFromTable(ui->comboBoxIndiceStd,"indices_estandar");
    DB->getStringFieldFromTableforID("indices_estandar", "ID", ui->comboBoxIndiceStd->currentText(), "DESCRIPCION", &descripcion);
    ui->labelIndiceStd->setText(descripcion);

    ui->comboBoxPeriodo->addItem("Dia");
    ui->comboBoxPeriodo->addItem("Semana");
    ui->comboBoxPeriodo->addItem("Mes");
    ui->comboBoxPeriodo->addItem("Anio");
}

IndicesWindow::~IndicesWindow()
{
    delete ui;
}

void IndicesWindow::on_addIndiceButton_clicked()
{
    extern DBHandler *DB;
    bool result=false;
    result=DB->insertIntoIndices(ui->lineEditID->text(), ui->comboBoxIndiceStd->currentText(), ui->lineEditPlazo->text().toInt(), ui->comboBoxPeriodo->currentText());

    if (result)
    {
        QMessageBox::information(0,"Insercion en Base de Datos","Indice insertado con exito");
        DB->getDataFromTable(ui->tableWidget,"indices");
    }
}

void IndicesWindow::on_delIndiceButton_clicked()
{
    extern DBHandler *DB;
    int numDeals=0;
    QString mensaje = "Se va a proceder a borrar el indice: ";
    QStringList *lista;
    if (!ui->tableWidget->rowCount())
    {
        QMessageBox::warning(0,"Eliminar","La tabla esta vacia");
        return;
    }
    int row=ui->tableWidget->currentRow();
    QString indice=ui->tableWidget->item(row,0)->text();
    mensaje.append(indice);

    switch( QMessageBox::warning( this, "Finanval",mensaje,"Si","No", 0, 0, 1 ))
    {
    case 0:
        lista=DB->getDealsContainingIndex(indice);

        numDeals=lista->count();
        if (numDeals)
        {
            mensaje="El indice esta incluido en los contratos: ";
            for (int i=0; i<numDeals; i++)
            {
                if (i)
                    mensaje.append(", ");
                mensaje.append(lista->value(i));
            }
            mensaje.append("\nElimine o sustituya este indice en las contratos antes de proceder");
            QMessageBox::critical(0,"Error",mensaje);
       }
       else
       {
           if (DB->deleteIndice(indice)) //Si todo fue bien repintamos la tabla
               DB->getDataFromTable(ui->tableWidget, "indices");
       }
       break;
    case 1:
        return;
        break;
    }

}

void IndicesWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}

void IndicesWindow::on_cleanButton_clicked()
{
    ui->lineEditID->setText("");
    ui->lineEditPlazo->setText("");
}

void IndicesWindow::on_comboBoxIndiceStd_currentIndexChanged(const QString &arg1)
{
    extern DBHandler *DB;
    QString descripcion;
    DB->getStringFieldFromTableforID("indices_estandar", "ID", arg1, "DESCRIPCION", &descripcion);
    ui->labelIndiceStd->setText(descripcion);
}

void IndicesWindow::on_pushButtonFijaciones_clicked()
{
    QString indice = ui->tableWidget->item(ui->tableWidget->currentRow(),0)->text();
    FijacionesWindow *ventanaFijaciones = new FijacionesWindow(indice, this);
    ventanaFijaciones->exec();
}

void IndicesWindow::on_tableWidget_cellClicked(int row, int column)
{
    ui->tableWidget->selectRow(row);
}
