//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "informewindow.h"
#include "ui_informewindow.h"
#include "nightcharts.h"
#include <QPainter>
#include "DBTools.h"
#include "valoracion.h"
#include <QMessageBox>


InformeWindow::InformeWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::InformeWindow)
{
    extern DBHandler *DB;
    extern QString defaultCurrency;
    DB->getDefaultCurrency(&defaultCurrency);
    QStringList deals, errores;
    double NPV=0., delta=0., gamma=0., rho=0., theta=0., vega=0.;
    QPalette palDelta;
    QPalette palGamma;
    QPalette palNPV;
    QPalette palRho;
    QPalette palTheta;
    QPalette palVega;
    QPalette palVN;

    ui->setupUi(this);

    ui->labelCurr->setText(defaultCurrency);
    DB->getOrderedDeals(ui->tableWidgetVencimientos);

    ui->lineEditDelta->setReadOnly(true);
    ui->lineEditGamma->setReadOnly(true);
    ui->lineEditNPV->setReadOnly(true);
    ui->lineEditRho->setReadOnly(true);
    ui->lineEditTheta->setReadOnly(true);
    ui->lineEditVega->setReadOnly(true);

    DB->getDeals(&deals);

    getMultipleValoration(deals,&NPV, &delta, &gamma, &rho, &theta, &vega, &errores);
    if (errores.count())
        QMessageBox::warning(0,"Error en los contratos","Revise los siguientes contratos:\n"+errores.join("\n"));

    if (delta<0)
        palDelta.setColor(QPalette::Text, Qt::red);
    else
        palDelta.setColor(QPalette::Text, Qt::blue);

    if (gamma<0)
        palGamma.setColor(QPalette::Text, Qt::red);
    else
        palGamma.setColor(QPalette::Text, Qt::blue);

    if (NPV<0)
        palNPV.setColor(QPalette::Text, Qt::red);
    else
        palNPV.setColor(QPalette::Text, Qt::blue);

    if (rho<0)
        palRho.setColor(QPalette::Text, Qt::red);
    else
        palRho.setColor(QPalette::Text, Qt::blue);

    if (theta<0)
        palTheta.setColor(QPalette::Text, Qt::red);
    else
        palTheta.setColor(QPalette::Text, Qt::blue);

    if (vega<0)
        palVega.setColor(QPalette::Text, Qt::red);
    else
        palVega.setColor(QPalette::Text, Qt::blue);


    ui->lineEditDelta->setPalette(palDelta);
    ui->lineEditGamma->setPalette(palGamma);
    ui->lineEditNPV->setPalette(palNPV);
    ui->lineEditRho->setPalette(palRho);
    ui->lineEditTheta->setPalette(palTheta);
    ui->lineEditVega->setPalette(palVega);


    ui->lineEditDelta->setText(QString("%1").arg(delta));
    ui->lineEditGamma->setText(QString("%1").arg(gamma));
    ui->lineEditNPV->setText(QString("%1").arg(NPV));
    ui->lineEditRho->setText(QString("%1").arg(rho));
    ui->lineEditTheta->setText(QString("%1").arg(theta));
    ui->lineEditVega->setText(QString("%1").arg(vega));

}

InformeWindow::~InformeWindow()
{
    delete ui;
}
