//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "irswindow.h"
#include "ui_irswindow.h"
#include "DBTools.h"
#include "valoracion.h"
#include <QPalette>
#include <QKeyEvent>
#include <ql/quantlib.hpp>
#include <QMessageBox>
#include <QPalette>
#include <flujoswindow.h>
#include <QTableWidget>
#include <iostream>

IRSWindow::IRSWindow(int ID, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::IRSWindow)
{
    extern DBHandler *DB;
    ui->setupUi(this);
    dealID=ID;


    //Setting labels
    QString *familia=DB->getFamilyFromProduct("IRS");
    ui->labelFamilia->setText(*familia);
    ui->labelFamilia->setStyleSheet("QLabel { color : blue; }");
    ui->labelProducto->setText("IRS");
    ui->labelProducto->setStyleSheet("QLabel { color : blue; }");
    if (!dealID)
        ui->labelIDContrato->setText("Nuevo");
    else
    {
        QString mID=QString("%1").arg(dealID);
        ui->labelIDContrato->setText(mID);
    }
    ui->labelIDContrato->setStyleSheet("QLabel { color : red; }");

    //Setting comboBox
    DB->getDataFromTable(ui->comboBoxConvencinFestivosFija,"convenciones_festivos");
    DB->getDataFromTable(ui->comboBoxConvencionFestivosFlotante,"convenciones_festivos");
    DB->getDataFromTable(ui->comboBoxConvencionCalendarioFija,"convenciones_calendario");
    DB->getDataFromTable(ui->comboBoxConvencionCalendarioFlotante,"convenciones_calendario");
    DB->getDataFromTable(ui->comboBoxDivisa,"divisas");
    DB->getDataFromTable(ui->comboBoxFrecuenciaFija,"frecuencias");
    DB->getDataFromTable(ui->comboBoxFrecuenciaFlotante,"frecuencias");
    DB->getDataFromTable(ui->comboBoxIndice,"indices");
    DB->getDataFromTable(ui->comboBoxCurvaDescuento,"curvas");
    DB->getDataFromTable(ui->comboBoxCurvaForward,"curvas");
    ui->comboBoxRol->addItem("Pagador");
    ui->comboBoxRol->addItem("Recibidor");

    //Setting double input
    ui->doubleSpinBoxNominal->setRange(0.,999999999999.99);
    ui->doubleSpinBoxNominal->setDecimals(2);
    ui->doubleSpinBoxSpread->setRange(0.,999999999999.99);
    ui->doubleSpinBoxSpread->setDecimals(2);
    ui->doubleSpinBoxTipoFijo->setRange(0.,999.99);
    ui->doubleSpinBoxTipoFijo->setDecimals(2);



    if (dealID)
    {
        DB->getIRS( dealID, ui->doubleSpinBoxNominal, ui->dateEditSettlement, ui->dateEditMaturity,
                    ui->comboBoxDivisa, ui->comboBoxRol, ui->doubleSpinBoxSpread,
                    ui->checkBoxIsFwd, ui->dateEditFwdStart,
                    ui->comboBoxFrecuenciaFija, ui->comboBoxConvencinFestivosFija,
                    ui->comboBoxConvencionCalendarioFija, ui->doubleSpinBoxTipoFijo,
                    ui->comboBoxCurvaDescuento, ui->comboBoxFrecuenciaFlotante,
                    ui->comboBoxConvencionFestivosFlotante, ui->comboBoxConvencionCalendarioFlotante,
                    ui->comboBoxIndice, ui->comboBoxCurvaForward);

    }

    //Settings results

    ui->lineEditNPV->setReadOnly(true);
    ui->lineEditFairSpread->setReadOnly(true);
    ui->lineEditFairRate->setReadOnly(true);

    if (!ui->checkBoxIsFwd->isChecked())
    {
        ui->labelForwardStart->hide();
        ui->dateEditFwdStart->hide();
        ui->pushButtonDesdeCalFwdStart->hide();
    }

}

IRSWindow::~IRSWindow()
{
    delete ui;
}

void IRSWindow::on_pushButtonDesdeCalSettlement_clicked()
{
    ui->dateEditSettlement->setDate(ui->calendarWidget->selectedDate());
}

void IRSWindow::on_pushButtonDesdeCalMaturity_clicked()
{
        ui->dateEditMaturity->setDate(ui->calendarWidget->selectedDate());
}

void IRSWindow::on_pushButtonDesdeCalFwdStart_clicked()
{
    ui->dateEditFwdStart->setDate(ui->calendarWidget->selectedDate());
}

void IRSWindow::on_checkBoxIsFwd_clicked()
{
    if (ui->checkBoxIsFwd->isChecked())
    {
        ui->labelForwardStart->show();
        ui->dateEditFwdStart->show();
        ui->pushButtonDesdeCalFwdStart->show();
    }
    else
    {
        ui->labelForwardStart->hide();
        ui->dateEditFwdStart->hide();
        ui->pushButtonDesdeCalFwdStart->hide();
    }

}

void IRSWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}

void IRSWindow::on_pushButtonValorar_clicked()
{

    QuantLib::VanillaSwap *irs=NULL;
    double NPV=0.0;
    double fairRate=0.0;
    double fairSpread=0.0;
    try {
        irs=getIRS(ui->doubleSpinBoxNominal->value(), ui->dateEditSettlement->date(), ui->dateEditMaturity->date(),
                ui->comboBoxDivisa->currentText(), ui->comboBoxRol->currentText(), ui->doubleSpinBoxSpread->value(),
                ui->checkBoxIsFwd->isChecked(), ui->dateEditFwdStart->date(),
                ui->comboBoxFrecuenciaFija->currentText(), ui->comboBoxConvencinFestivosFija->currentText(),
                ui->comboBoxConvencionCalendarioFija->currentText(), ui->doubleSpinBoxTipoFijo->value(),
                ui->comboBoxCurvaDescuento->currentText(), ui->comboBoxFrecuenciaFlotante->currentText(),
                ui->comboBoxConvencionFestivosFlotante->currentText(), ui->comboBoxConvencionCalendarioFlotante->currentText(),
                ui->comboBoxIndice->currentText(), ui->comboBoxCurvaForward->currentText());
        if (!irs)
            return;
        NPV = irs->NPV();
        fairRate = irs->fairRate();
        fairSpread = irs->fairSpread();
    }
    catch ( std :: exception & e)
    {
        QMessageBox::critical(0,"Error en valoracion",e.what());
        return ;
    } catch (...)
    {
        QMessageBox::critical(0,"Error en valoracion","Error desconocido");
        return ;
    }

    QPalette palNPV;
    QPalette palfairRate;
    QPalette palfairSpread;


    if (NPV<0)
        palNPV.setColor(QPalette::Text, Qt::red);
    else
        palNPV.setColor(QPalette::Text, Qt::blue);

    if (fairRate<0)
        palfairRate.setColor(QPalette::Text, Qt::red);
    else
        palfairRate.setColor(QPalette::Text, Qt::blue);

    if (fairSpread<0)
        palfairSpread.setColor(QPalette::Text, Qt::red);
    else
        palfairSpread.setColor(QPalette::Text, Qt::blue);

    ui->lineEditNPV->setPalette(palNPV);
    ui->lineEditFairRate->setPalette(palfairRate);
    ui->lineEditFairSpread->setPalette(palfairSpread);

    ui->lineEditNPV->setText(QString("%1").arg(NPV));
    ui->lineEditFairRate->setText(QString("%1").arg(fairRate));
    ui->lineEditFairSpread->setText(QString("%1").arg(fairSpread));


}

void IRSWindow::on_pushButtonFlujosFijos_clicked()
{
    QuantLib::VanillaSwap *irs=NULL;

    try
    {
        irs=getIRS(ui->doubleSpinBoxNominal->value(), ui->dateEditSettlement->date(), ui->dateEditMaturity->date(),
                ui->comboBoxDivisa->currentText(), ui->comboBoxRol->currentText(), ui->doubleSpinBoxSpread->value(),
                ui->checkBoxIsFwd->isChecked(), ui->dateEditFwdStart->date(),
                ui->comboBoxFrecuenciaFija->currentText(), ui->comboBoxConvencinFestivosFija->currentText(),
                ui->comboBoxConvencionCalendarioFija->currentText(), ui->doubleSpinBoxTipoFijo->value(),
                ui->comboBoxCurvaDescuento->currentText(), ui->comboBoxFrecuenciaFlotante->currentText(),
                ui->comboBoxConvencionFestivosFlotante->currentText(), ui->comboBoxConvencionCalendarioFlotante->currentText(),
                ui->comboBoxIndice->currentText(), ui->comboBoxCurvaForward->currentText());
        if (!irs)
            return;
    }
    catch ( std :: exception & e)
    {
        QMessageBox::critical(0,"Error en valoracion",e.what());
        return ;
    } catch (...)
    {
        QMessageBox::critical(0,"Error en valoracion","Error desconocido");
        return ;
    }
    QuantLib::Schedule fixedSchedule=irs->fixedSchedule();
    QString tituloTabla("FECHA DE PAGO");
    QTableWidget *tabla = new QTableWidget;
    tabla->setColumnCount(1);
    tabla->setHorizontalHeaderLabels(tituloTabla.split(";"));
    int numFlujos=fixedSchedule.size();
    tabla->setRowCount(numFlujos);
    int dia;
    int mes;
    int anio;

    for (int i=0; i<numFlujos; i++)
    {
        dia=fixedSchedule[i].dayOfMonth();
        mes=(int)fixedSchedule[i].month();
        anio=fixedSchedule[i].year();
        QString dateString = QString(tr("%1/%2/%3")).arg(dia).arg(mes).arg(anio);
        tabla->setItem(i, 0, new QTableWidgetItem(dateString));
    }
    FlujosWindow *ventanaFlujos = new FlujosWindow(tabla, this);
    ventanaFlujos->exec();
}

void IRSWindow::on_pushButtonFlujosFlotantes_clicked()
{
    QuantLib::VanillaSwap *irs=NULL;

    try
    {

        irs=getIRS(ui->doubleSpinBoxNominal->value(), ui->dateEditSettlement->date(), ui->dateEditMaturity->date(),
                ui->comboBoxDivisa->currentText(), ui->comboBoxRol->currentText(), ui->doubleSpinBoxSpread->value(),
                ui->checkBoxIsFwd->isChecked(), ui->dateEditFwdStart->date(), ui->comboBoxFrecuenciaFija->currentText(),
                ui->comboBoxConvencinFestivosFija->currentText(),
                ui->comboBoxConvencionCalendarioFija->currentText(), ui->doubleSpinBoxTipoFijo->value(),
                ui->comboBoxCurvaDescuento->currentText(), ui->comboBoxFrecuenciaFlotante->currentText(),
                ui->comboBoxConvencionFestivosFlotante->currentText(), ui->comboBoxConvencionCalendarioFlotante->currentText(),
                ui->comboBoxIndice->currentText(), ui->comboBoxCurvaForward->currentText());
        if (!irs)
            return;
    }
    catch ( std :: exception & e)
    {
        QMessageBox::critical(0,"Error en valoracion",e.what());
        return ;
    } catch (...)
    {
        QMessageBox::critical(0,"Error en valoracion","Error desconocido");
        return ;
    }
    QuantLib::Schedule floatingSchedule=irs->floatingSchedule();
    QString tituloTabla("FECHA DE PAGO");
    QTableWidget *tabla = new QTableWidget;
    tabla->setColumnCount(1);
    tabla->setHorizontalHeaderLabels(tituloTabla.split(";"));
    int numFlujos=floatingSchedule.size();
    tabla->setRowCount(numFlujos);
    int dia;
    int mes;
    int anio;

    for (int i=0; i<numFlujos; i++)
    {
        dia=floatingSchedule[i].dayOfMonth();
        mes=(int)floatingSchedule[i].month();
        anio=floatingSchedule[i].year();
        QString dateString = QString(tr("%1/%2/%3")).arg(dia).arg(mes).arg(anio);
        tabla->setItem(i, 0, new QTableWidgetItem(dateString));
    }
    FlujosWindow *ventanaFlujos = new FlujosWindow(tabla, this);
    ventanaFlujos->exec();

}

void IRSWindow::on_buttonBox_clicked(QAbstractButton *button)
{
    extern DBHandler *DB;
    bool deleted=false, inserted=false;
    if (button==ui->buttonBox->button(QDialogButtonBox::Save))
    {

        if (dealID)
        {
            deleted=DB->deleteDeal(dealID);
            if (!deleted)
            {
                QMessageBox::critical(0,"Guardar","No se pudo guardar el IRS");
                return;
            }

        }
        inserted=DB->insertIRS(ui->doubleSpinBoxNominal->value(), ui->dateEditSettlement->date(), ui->dateEditMaturity->date(),
                      ui->comboBoxDivisa->currentText(), ui->comboBoxRol->currentText(), ui->doubleSpinBoxSpread->value(),
                      ui->checkBoxIsFwd->isChecked(), ui->dateEditFwdStart->date(), ui->comboBoxFrecuenciaFija->currentText(),
                      ui->comboBoxConvencinFestivosFija->currentText(),
                      ui->comboBoxConvencionCalendarioFija->currentText(), ui->doubleSpinBoxTipoFijo->value(),
                      ui->comboBoxCurvaDescuento->currentText(), ui->comboBoxFrecuenciaFlotante->currentText(),
                      ui->comboBoxConvencionFestivosFlotante->currentText(), ui->comboBoxConvencionCalendarioFlotante->currentText(),
                      ui->comboBoxIndice->currentText(), ui->comboBoxCurvaForward->currentText(), &dealID);
        if (inserted)
        {
            QString msg= QString("Contrato guardado con exito, identificador %1").arg(dealID);
            QMessageBox::information(0,"Guardar",msg);
        }
        else
            QMessageBox::critical(0,"Guardar","No se pudo guardar el IRS");
    }
    else
        this->close();
}
