//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QStandardItemModel>
#include <QListView>
#include <QTableView>
#include <QSplitter>
#include <iostream>
#include "dealswindow.h"
#include "depositoswindow.h"
#include "futuroswindow.h"
#include "fraswindow.h"
#include "swapswindow.h"
#include "curvaswindow.h"
#include "indiceswindow.h"
#include "divisaswindow.h"
#include "valoracion.h"
#include <calendariowindow.h>
#include <tiposcambiowindow.h>
#include <informewindow.h>
#include <aboutwindow.h>
#include <contenidowindow.h>
#include <QMessageBox>
#include <QFileDialog>




MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    extern DBHandler *DB;
    DB = new DBHandler;
    DB->conectar();
    DB->updateCartera(ui->arbolProducts);
}

MainWindow::~MainWindow()
{
    ui->arbolProducts->clear();
    delete ui;
    extern DBHandler *DB;
    delete DB;

}

void MainWindow::on_actionSalir_triggered()
{
    exit(0);
}

void MainWindow::on_actionPricing_triggered()
{
    extern DBHandler *DB;

    DealsWindow *ventanaDeals = new DealsWindow(this);
    ventanaDeals->exec();
    DB->updateCartera(ui->arbolProducts);

}

void MainWindow::on_actionDepositos_triggered()
{
    DepositosWindow *ventanaDepositos = new DepositosWindow(this);
    ventanaDepositos->exec();

}

void MainWindow::on_actionFuturos_triggered()
{
    FuturosWindow *ventanaFuturos = new FuturosWindow(this);
    ventanaFuturos->exec();
}

void MainWindow::on_actionCurvas_triggered()
{
    CurvasWindow *ventanaCurvas = new CurvasWindow(this);
    ventanaCurvas->exec();
}

void MainWindow::on_actionFRAs_triggered()
{
    FrasWindow *ventanaFras = new FrasWindow(this);
    ventanaFras->exec();
}

void MainWindow::on_actionSwaps_triggered()
{
    SwapsWindow *ventanaSwaps = new SwapsWindow(this);
    ventanaSwaps->exec();
}

void MainWindow::on_actionIndices_triggered()
{
    IndicesWindow *ventanaIndices = new IndicesWindow(this);
    ventanaIndices->exec();
}

void MainWindow::on_actionDivisas_triggered()
{
    DivisasWindow *ventanaDivisas= new DivisasWindow(this);
    ventanaDivisas->exec();
}

void MainWindow::on_actionCalendarios_triggered()
{
    CalendarioWindow *ventanaCalendario= new CalendarioWindow(this);
    ventanaCalendario->exec();
}

void MainWindow::on_botonValorar_clicked()
{
    extern DBHandler *DB;
    extern QString defaultCurrency;
    double NPV=0.;
    double delta, gamma, rho, theta, vega;
    QStringList deals, errores;
    QString raiz;
    QList<QTreeWidgetItem*> lista=ui->arbolProducts->selectedItems();
    int numItems=0;
    numItems=lista.count();
    for (int i=0; i<numItems; i++)
    {
        QTreeWidgetItem *item=lista.at(i);
        if (i==0)
            raiz=item->text(0);
        if (item->type()==TIPO_PRODUCTO)
            deals.append(item->text(0));
        else
        {
            QList<QTreeWidgetItem*> listaHijos=item->takeChildren();
            for (int hijo=0; hijo<listaHijos.count(); hijo++)
            {
                QTreeWidgetItem *itemHijo=listaHijos.at(hijo);
                if (itemHijo->type()==TIPO_PRODUCTO)
                    deals.append(itemHijo->text(0));
                else
                {
                    QList<QTreeWidgetItem*> listaNietos=itemHijo->takeChildren();
                    for (int nieto=0; nieto<listaNietos.count(); nieto++)
                    {
                        QTreeWidgetItem *itemNieto=listaNietos.at(nieto);
                        if (itemNieto->type()==TIPO_PRODUCTO)
                            deals.append(itemNieto->text(0));
                        delete itemNieto;
                    }
                }
                delete itemHijo;
            }
        }
        delete item;
    }
    getMultipleValoration(deals,&NPV, &delta, &gamma, &rho, &theta, &vega, &errores);
    if (errores.count())
        QMessageBox::warning(0,"Error en los contratos","Revise los siguientes contratos:\n"+errores.join("\n"));
    QString txt=QString("Net Present Value: %1").arg(NPV);
    QMessageBox::information(0,"Resultado de la valoracion agregada",txt+" "+defaultCurrency);

    DB->updateCartera(ui->arbolProducts);
}

void MainWindow::on_actionTipos_de_cambio_triggered()
{
    TiposCambioWindow *ventanaTiposCambio= new TiposCambioWindow(this);
    ventanaTiposCambio->exec();
}

void MainWindow::on_actionSimulaci_n_triggered()
{
    InformeWindow *ventanaInforme = new InformeWindow(this);
    ventanaInforme->exec();
}

void MainWindow::on_actionExportar_triggered()
{
    extern DBHandler *DB;
    QString fileName = QFileDialog::getSaveFileName();
    DB->exportDealsToFile(fileName);

}

void MainWindow::on_actionAcerca_de_triggered()
{
    AboutWindow *ventanaAbout= new AboutWindow(this);
    ventanaAbout->exec();
}

void MainWindow::on_actionContenido_triggered()
{
    ContenidoWindow *ventanaContenido= new ContenidoWindow(this);
    ventanaContenido->exec();

}
