//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "nuevacurvawindow.h"
#include "ui_nuevacurvawindow.h"
#include <ql/quantlib.hpp>
#include <QMessageBox>
#include <QModelIndex>
#include <QKeyEvent>

using namespace QuantLib;

NuevaCurvaWindow::NuevaCurvaWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::NuevaCurvaWindow)
{
    ui->setupUi(this);
    extern DBHandler *DB;
    int fieldno=0;
    QSqlQuery *queryDivisas=NULL;
    queryDivisas=DB->getDataFromTable("divisas");
    while (queryDivisas->next())
    {
        fieldno = queryDivisas->record().indexOf("CURRENCY_ISO");
        QString txt(queryDivisas->value(fieldno).toString());
        ui->comboDivisa->addItem(txt);
    }
    if (queryDivisas)
        delete queryDivisas;

    ui->comboProductos->addItem("Depositos");
    ui->comboProductos->addItem("Futuros");
    ui->comboProductos->addItem("Fras");
    ui->comboProductos->addItem("Swaps");
    DB->getDataFromTable(ui->tablaProductos,"depositos");
    actualProductRow=0;
    ui->tablaProductos->selectRow(actualProductRow);
    ui->doubleSpinBoxTolerancia->setRange(0.000000000000001,1.0);
    ui->doubleSpinBoxTolerancia->setDecimals(15);
    ui->doubleSpinBoxTolerancia->setSingleStep(0.000000000000001);
    DB->getDataFromTable(ui->comboBoxConvCal,"convenciones_calendario");
}

NuevaCurvaWindow::~NuevaCurvaWindow()
{
    delete ui;
}

void NuevaCurvaWindow::on_comboProductos_activated(const QString &arg1)
{
    extern DBHandler *DB;
    if (arg1=="Depositos")
        DB->getDataFromTable(ui->tablaProductos,"depositos");
    if (arg1=="Futuros")
        DB->getDataFromTable(ui->tablaProductos,"futuros");
    if (arg1=="Fras")
        DB->getDataFromTable(ui->tablaProductos,"fras");
    if (arg1=="Swaps")
        DB->getDataFromTable(ui->tablaProductos,"swaps");

}

void NuevaCurvaWindow::on_tablaProductos_cellClicked(int row, int column)
{
    ui->tablaProductos->selectRow(row);
    actualProductRow=row;

}

void NuevaCurvaWindow::on_pushButton_clicked()
{
    extern DBHandler *DB;
    int index=0;
    QString tituloCurvas;
    QString producto = ui->comboProductos->currentText();
    QString tipoProducto;
    QString startDate;
    QString divisaProd;
    QString divisa;
    QString plazo;
    QString periodo;
    QString DF("0.0");
    QString marketQuote;
    int row = actualProductRow;

    if (row<0)
    {
        QMessageBox::critical(0,"Error en seleccion","Seleccione una fila en la tabla de productos");
        return;
    }
    QString idProducto=ui->tablaProductos->item(row,0)->text();
    int numRows=ui->tablaCurvas->rowCount();
    for (int i=0; i<numRows; i++)
    {
        if (ui->tablaCurvas->item(i,0)->text()==idProducto)
        {
            QMessageBox::warning(0,"Error en seleccionn","Producto ya insertado");
            return;
        }

    }

    if (producto=="Depositos")
        tipoProducto="Deposito";
    else if (producto=="Fras")
        tipoProducto="Fra";
    else if (producto=="Swaps")
        tipoProducto="Swap";
    else if(producto=="Futuros")
        tipoProducto="Futuro";
    else
        tipoProducto="Unknown";

    QSqlQuery *query=DB->getProducto(producto.toLower(), idProducto);
    query->next(); //Solo puede haber un registro (primary key)
    index=query->record().indexOf("QUOTE");
    marketQuote=query->value(index).toString();
    index=query->record().indexOf("CURRENCY_ISO");
    divisaProd=query->value(index).toString();

    divisa=ui->comboDivisa->currentText();
    if (divisa!=divisaProd)
    {
        QMessageBox::warning(0,"Error en seleccion","Producto con divisa diferente a la curva");
        delete query;
        return;
    }

    int spotCurrency=DB->getCurrencySpotSchedule(divisa);

    Calendar calendar = TARGET();
    Date settlementDate=QuantLib::Date::todaysDate();

    // must be a business day
    settlementDate = calendar.adjust(settlementDate);

    Integer fixingDays = spotCurrency;
    Date todaysDate = calendar.advance(settlementDate, -fixingDays, Days);
    int dia=todaysDate.dayOfMonth();
    int mes=todaysDate.month();
    int ano=todaysDate.year();
    startDate = QString("%1/%2/%3").arg(dia).arg(mes).arg(ano);

    if (producto=="Depositos")
    {
        index=query->record().indexOf("PLAZO");
        plazo=query->value(index).toString();
        index=query->record().indexOf("PERIODO");
        periodo=query->value(index).toString();
    }
    else if (producto=="Futuros")
    {
        index=query->record().indexOf("MONTHS");
        plazo=query->value(index).toString();
        periodo="Mes";
    }
    else if (producto=="Fras")
    {
        index=query->record().indexOf("MONTHSTOSTART");
        int start=query->value(index).toInt();
        index=query->record().indexOf("MONTHSTOEND");
        int end=query->value(index).toInt();
        QString newPlazo = QString("%1").arg(start+end);
        plazo=newPlazo;
        periodo="Mes";
    }
    else if (producto=="Swaps")
    {
        index=query->record().indexOf("PLAZO");
        plazo=query->value(index).toString();
        index=query->record().indexOf("PERIODO");
        periodo=query->value(index).toString();
    }
    delete query;

    ui->tablaCurvas->setColumnCount(8);
    tituloCurvas.append("PRODUCT_ID;TIPO;DIVISA;START DATE;PLAZO;PERIODO;DISCOUNT FACTOR;MARKET QUOTE");
    ui->tablaCurvas->setHorizontalHeaderLabels(tituloCurvas.split(";"));
    row=ui->tablaCurvas->rowCount();
    ui->tablaCurvas->setRowCount(row+1);
    ui->tablaCurvas->setItem(row, 0, new QTableWidgetItem(idProducto));
    ui->tablaCurvas->setItem(row, 1, new QTableWidgetItem(tipoProducto));
    ui->tablaCurvas->setItem(row, 2, new QTableWidgetItem(divisa));
    ui->tablaCurvas->setItem(row, 3, new QTableWidgetItem(startDate));
    ui->tablaCurvas->setItem(row, 4, new QTableWidgetItem(plazo));
    ui->tablaCurvas->setItem(row, 5, new QTableWidgetItem(periodo));
    ui->tablaCurvas->setItem(row, 6, new QTableWidgetItem(DF));
    ui->tablaCurvas->setItem(row, 7, new QTableWidgetItem(marketQuote));
    ui->tablaCurvas->item(row,6)->setTextColor("Blue");
    ui->tablaCurvas->resizeColumnsToContents();

}

void NuevaCurvaWindow::on_tablaCurvas_clicked(const QModelIndex &index)
{
    ui->tablaCurvas->selectRow(index.row());
}

void NuevaCurvaWindow::on_pushButtonEliminarDeCurva_clicked()
{
    if (ui->tablaCurvas->rowCount())
        ui->tablaCurvas->removeRow(ui->tablaCurvas->currentRow());
}

void NuevaCurvaWindow::on_buttonBox_rejected()
{
    this->reject();
}

void NuevaCurvaWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}

void NuevaCurvaWindow::on_buttonBox_accepted()
{
    extern DBHandler *DB;
    int numRows=0;
    bool ret=false;
    QString curvaID, divisa, convencionCal;
    QStringList listaProductos;
    double tolerancia=0.0;
    if (!ui->tablaCurvas->rowCount())
    {
        QMessageBox::warning(0,"Error en insercion","La curva esta vacia. Inserte productos.");
        return;
    }
    if (ui->nombreCurvaEdit->text()=="")
    {
        QMessageBox::warning(0,"Insercion","Es necesario un identificador de curva");
        return;
    }
    curvaID=ui->nombreCurvaEdit->text();
    divisa=ui->comboDivisa->currentText();
    tolerancia=ui->doubleSpinBoxTolerancia->value();
    convencionCal=ui->comboBoxConvCal->currentText();
    numRows=ui->tablaCurvas->rowCount();
    for (int i=0; i<numRows; i++)
    {
        listaProductos.append(ui->tablaCurvas->item(i,0)->text());
    }
    ret=DB->insertIntoCurvas(curvaID, divisa, tolerancia, convencionCal, listaProductos);
    if (ret)
    {
        this->accept();
    }
}
