//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "tiposcambiowindow.h"
#include "ui_tiposcambiowindow.h"
#include "curvaswindow.h"
#include "DBTools.h"
#include <QMessageBox>
#include <QValidator>
#include <QIntValidator>
#include <QDoubleValidator>
#include <QKeyEvent>

TiposCambioWindow::TiposCambioWindow(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::TiposCambioWindow)
{
    ui->setupUi(this);
    extern DBHandler *DB;
    extern QString defaultCurrency;
    int index=-1;

    initializing=true;   //para no activar los triggers sobre celdas de la tabla

    QValidator *val1=new QDoubleValidator(0,999999,8,ui->lineEditTipoCambio);
    ui->lineEditTipoCambio->setValidator(val1);

    DB->getDataFromTiposCambio(ui->tableWidget);

    DB->getDataFromTable(ui->comboBoxDefaultCurrency, "divisas");
    DB->getDefaultCurrency(&defaultCurrency);
    index = ui->comboBoxDefaultCurrency->findText(defaultCurrency);
    ui->comboBoxDefaultCurrency->setCurrentIndex(index);

    DB->getDataFromTable(ui->comboBoxDivisaDomestica, "divisas");
    DB->getDataFromTable(ui->comboBoxDivisaForanea, "divisas");

    initializing=false;
}

TiposCambioWindow::~TiposCambioWindow()
{
    delete ui;
}

void TiposCambioWindow::on_addTipoCambioButton_clicked()
{
    extern DBHandler *DB;
    bool result=false;
    if (ui->comboBoxDivisaDomestica->currentText()==ui->comboBoxDivisaForanea->currentText())
    {
        QMessageBox::critical(0,"Error en insercion","La divisa domestica debe ser diferente de la foranea");
        return;
    }
    result=DB->insertIntoTiposCambio(ui->comboBoxDivisaDomestica->currentText(), ui->comboBoxDivisaForanea->currentText(), ui->lineEditTipoCambio->text().toDouble());
    if (result)
        DB->getDataFromTiposCambio(ui->tableWidget);
}

void TiposCambioWindow::on_delTipoCambioButton_clicked()
{
    extern DBHandler *DB;
    extern QString defaultCurrency;

    QString mensaje = "Se va a proceder a borrar el tipo de cambio: ";
    if (!ui->tableWidget->rowCount())
    {
        QMessageBox::warning(0,"Eliminar","La tabla esta vacia");
        return;
    }
    int row=ui->tableWidget->currentRow();
    QString domestica=ui->tableWidget->item(row,0)->text();
    QString foranea=ui->tableWidget->item(row,1)->text();
    mensaje.append(domestica+"-"+foranea);

    switch( QMessageBox::warning( this, "Finanval",mensaje,"Si","No", 0, 0, 1 ))
    {
    case 0:
        if ((defaultCurrency==domestica) || (defaultCurrency==foranea))
        {

            switch (QMessageBox::critical( this, "Finanval","La divisa por defecto esta incluida en este par\nNo podra valorar productos que la contengan","Ok", "Cancel", 0, 0, 1 ))
            {
            case 0:
                if (DB->deleteTipoCambio(domestica, foranea)) //Si todo fue bien repintamos la tabla
                    DB->getDataFromTiposCambio(ui->tableWidget);
                return;
            case 1:
                return;
            }
        }
        else
        {
            if (DB->deleteTipoCambio(domestica, foranea)) //Si todo fue bien repintamos la tabla
                DB->getDataFromTiposCambio(ui->tableWidget);
        }
        break;
    case 1:
        return;
        break;
    }

}

void TiposCambioWindow::keyPressEvent(QKeyEvent *e)
{
    if(e->key()!=Qt::Key_Return)
        QDialog::keyPressEvent(e);
}

void TiposCambioWindow::on_tableWidget_cellChanged(int row, int column)
{
    extern DBHandler *DB;

    if ((!initializing) && (column==ui->tableWidget->columnCount()-1))
    {
        QString domestica(ui->tableWidget->item(row,0)->text());
        QString foranea(ui->tableWidget->item(row,1)->text());
        DB->updateTipoCambio(domestica, foranea, ui->tableWidget->item(row,column)->text().toDouble());
    }
}

void TiposCambioWindow::on_cleanButton_clicked()
{
    ui->lineEditTipoCambio->setText("");
}

void TiposCambioWindow::on_refrescarButton_clicked()
{
    extern DBHandler *DB;
    DB->getDataFromTiposCambio(ui->tableWidget);
}


void TiposCambioWindow::on_comboBoxDefaultCurrency_currentIndexChanged(const QString &arg1)
{
    extern DBHandler *DB;
    extern QString defaultCurrency;
    if (initializing)
        return;

    defaultCurrency=arg1;
    DB->updateDefaultCurrency(defaultCurrency);
}
