//****************************************************************************//
// Manuel Martínez Liébana                                                    //
// TFC - Ingeniería Técnica Informática de Gestión                            //
// UOC - 2012                                                                 //
//****************************************************************************//

#include "valoracion.h"
#include <QDebug>
#include "DBTools.h"
#include <vector>
#include <QMessageBox>

#define VERBOSE

using namespace std;

//------------------------------------------------------------------------------
QuantLib::VanillaSwap::Type getSwapType(QString tipo)
{
    if (tipo=="Pagador")
        return QuantLib::VanillaSwap::Payer;
    return  QuantLib::VanillaSwap::Receiver;
}

//------------------------------------------------------------------------------
QuantLib::Frequency getFrequency(QString frequency)
{
    using namespace QuantLib;
    if (frequency=="Once")
        return Once;
    else if (frequency=="Annual")
        return Annual;
    else if (frequency=="Semiannual")
        return Semiannual;
    else if (frequency=="EveryFourthMonth")
        return EveryFourthMonth;
    else if (frequency=="Quarterly")
        return Quarterly;
    else if (frequency=="Bimonthly")
        return Bimonthly;
    else if (frequency=="Monthly")
        return Monthly;
    else if (frequency=="EveryFourthWeek")
        return EveryFourthWeek;
    else if (frequency=="Biweekly")
        return Biweekly;
    else if (frequency=="Weekly")
        return Weekly;
    else if (frequency=="Daily")
        return Daily;
    return NoFrequency;
}
//------------------------------------------------------------------------------
void adjustCalendarHolidays(QString name, QuantLib::Calendar *cal)
{
    using namespace QuantLib;
    extern DBHandler *DB;
    int year, month, day;
    Date holidayDate;
    int numHolidays=0;
    QString holiday, addHolidays, delHolidays;
    QStringList fecha, lista;
    DB->getStringFieldFromTableforID("calendarios", "CALENDAR_ID", name, "ADDED_HOLIDAYS", &addHolidays);
    DB->getStringFieldFromTableforID("calendarios", "CALENDAR_ID", name, "REMOVED_HOLIDAYS", &delHolidays);

    //Primero añadimos los holidays pertinentes
    if ((addHolidays)!="")
    {
        lista=addHolidays.split(";");
        numHolidays=lista.count();
    }
    else
        numHolidays=0;
    for (int i=0; i<numHolidays; i++)
    {
        holiday=lista.at(i);
        if (holiday=="")
            continue;

        fecha=holiday.split("-");

        year=fecha.at(0).toInt();
        month=fecha.at(1).toInt();
        day=fecha.at(2).toInt();
        holidayDate=Date(day, (Month)month, year);
        cal->addHoliday(holidayDate);
    }
    //Despues eliminamos los holidays configurados para tal fin
    if ((delHolidays)!="")
    {
        lista=delHolidays.split(";");
        numHolidays=lista.count();
    }
    else
        numHolidays=0;
    for (int i=0; i<numHolidays; i++)
    {
        holiday=lista.at(i);
        if (holiday=="")
            continue;

        fecha=holiday.split("-");

        year=fecha.at(0).toInt();
        month=fecha.at(1).toInt();
        day=fecha.at(2).toInt();
        holidayDate=Date(day, (Month)month, year);
        cal->removeHoliday(holidayDate);
    }
}

//------------------------------------------------------------------------------
QuantLib::Calendar *getCalendar(QString name)
{
    using namespace QuantLib;

    Calendar *calendar = new Calendar();

    if ((name)=="TARGET")
    {
        (*calendar) = TARGET();
        adjustCalendarHolidays("TARGET", calendar);
        return calendar;
    }
    else if ((name)=="Argentina:Merval")
    {
        (*calendar) = Argentina(Argentina::Merval);
        adjustCalendarHolidays("Argentina:Merval", calendar);
        return calendar;
    }
    else if ((name)=="Australia")
    {
        (*calendar) = Australia();
        adjustCalendarHolidays("Australia", calendar);
        return calendar;
    }
    else if ((name)=="Brazil:Exchange")
    {
        (*calendar) = Brazil(Brazil::Exchange);
        adjustCalendarHolidays("Brazil:Exchange", calendar);
        return calendar;
    }
    else if ((name)=="Brazil:Settlement")
    {
        (*calendar) = Brazil(Brazil::Settlement);
        adjustCalendarHolidays("Brazil:Settlement", calendar);
        return calendar;
    }
    else if ((name)=="Canada:Settlement")
    {
        (*calendar) = Canada(Canada::Settlement);
        adjustCalendarHolidays("Canada:Settlement", calendar);
        return calendar;
    }
    else if ((name)=="Canada:TSX")
    {
        (*calendar) = Canada(Canada::TSX);
        adjustCalendarHolidays("Canada:TSX", calendar);
        return calendar;
    }
    else if ((name)=="China")
    {
        (*calendar) = China();
        adjustCalendarHolidays("China", calendar);
        return calendar;
    }
    else if ((name)=="CzechRepublic:PSE")
    {
        (*calendar) = CzechRepublic(CzechRepublic::PSE);
        adjustCalendarHolidays("CzechRepublic:PSE", calendar);
        return calendar;
    }
    else if ((name)=="Denmark")
    {
        (*calendar) = Denmark();
        adjustCalendarHolidays("Denmark", calendar);
        return calendar;
    }
    else if ((name)=="Finland")
    {
        (*calendar) = Finland();
        adjustCalendarHolidays("Finland", calendar);
        return calendar;
    }
    else if ((name)=="Germany:Eurex")
    {
        (*calendar) = Germany(Germany::Eurex);
        adjustCalendarHolidays("Germany:Eurex", calendar);
        return calendar;
    }
    else if ((name)=="Germany:FrankfurtStockExchange")
    {
        (*calendar) = Germany(Germany::FrankfurtStockExchange);
        adjustCalendarHolidays("Germany:FrankfurtStockExchange", calendar);
        return calendar;
    }
    else if ((name)=="Germany:Settlement")
    {
        (*calendar) = Germany(Germany::Settlement);
        adjustCalendarHolidays("Germany:Settlement", calendar);
        return calendar;
    }
    else if ((name)=="Germany:Xetra")
    {
        (*calendar) = Germany(Germany::Xetra);
        adjustCalendarHolidays("Germany:Xetra", calendar);
        return calendar;
    }
    else if ((name)=="HongKong:HKEx")
    {
        (*calendar) = HongKong(HongKong::HKEx);
        adjustCalendarHolidays("HongKong:HKEx", calendar);
        return calendar;
    }
    else if ((name)=="Hungary")
    {
        (*calendar) = Hungary();
        adjustCalendarHolidays("Hungary", calendar);
        return calendar;
    }
    else if ((name)=="Iceland:ICEX")
    {
        (*calendar) = Iceland(Iceland::ICEX);
        adjustCalendarHolidays("Iceland:ICEX", calendar);
        return calendar;
    }
    else if ((name)=="India:NSE")
    {
        (*calendar) = India(India::NSE);
        adjustCalendarHolidays("India:NSE", calendar);
        return calendar;
    }
    else if ((name)=="Indonesia:BEJ")
    {
        (*calendar) = Indonesia(Indonesia::BEJ);
        adjustCalendarHolidays("Indonesia:BEJ", calendar);
        return calendar;
    }
    else if ((name)=="Indonesia:JSX")
    {
        (*calendar) = Indonesia(Indonesia::JSX);
        adjustCalendarHolidays("Indonesia:JSX", calendar);
        return calendar;
    }
    else if ((name)=="Italy:Exchange")
    {
        (*calendar) = Italy(Italy::Exchange);
        adjustCalendarHolidays("Italy:Exchange", calendar);
        return calendar;
    }
    else if ((name)=="Italy:Settlement")
    {
        (*calendar) = Italy(Italy::Settlement);
        adjustCalendarHolidays("Italy:Settlement", calendar);
        return calendar;
    }
    else if ((name)=="Japan")
    {
        (*calendar) = Japan();
        adjustCalendarHolidays("Japan", calendar);
        return calendar;
    }
    else if ((name)=="Mexico:BMV")
    {
        (*calendar) = Mexico(Mexico::BMV);
        adjustCalendarHolidays("Mexico:BMV", calendar);
        return calendar;
    }
    else if ((name)=="NewZealand")
    {
        (*calendar) = NewZealand();
        adjustCalendarHolidays("NewZealand", calendar);
        return calendar;
    }
    else if ((name)=="Norway")
    {
        (*calendar) = Norway();
        adjustCalendarHolidays("Norway", calendar);
        return calendar;
    }
    else if ((name)=="Poland")
    {
        (*calendar) = Poland();
        adjustCalendarHolidays("Poland", calendar);
        return calendar;
    }
    else if ((name)=="SaudiArabia:Tadawul")
    {
        (*calendar) = SaudiArabia(SaudiArabia::Tadawul);
        adjustCalendarHolidays("SaudiArabia:Tadawul", calendar);
        return calendar;
    }
    else if ((name)=="Singapore:SGX")
    {
        (*calendar) = Singapore(Singapore::SGX);
        adjustCalendarHolidays("Singapore:SGX", calendar);
        return calendar;
    }
    else if ((name)=="Slovakia:BSSE")
    {
        (*calendar) = Slovakia(Slovakia::BSSE);
        adjustCalendarHolidays("Slovakia:BSSE", calendar);
        return calendar;
    }
    else if ((name)=="SouthAfrica")
    {
        (*calendar) = SouthAfrica();
        adjustCalendarHolidays("Africa", calendar);
        return calendar;
    }
    else if ((name)=="SouthKorea:KRX")
    {
        (*calendar) = SouthKorea(SouthKorea::KRX);
        adjustCalendarHolidays("SouthKorea:KRX", calendar);
        return calendar;
    }
    else if ((name)=="Sweden")
    {
        (*calendar) = Sweden();
        adjustCalendarHolidays("Sweden", calendar);
        return calendar;
    }
    else if ((name)=="Switzerland")
    {
        (*calendar) = Switzerland();
        adjustCalendarHolidays("Switzerland", calendar);
        return calendar;
    }
    else if ((name)=="Taiwan:TSEC")
    {
        (*calendar) = Taiwan(Taiwan::TSEC);
        adjustCalendarHolidays("Taiwan:TSEC", calendar);
        return calendar;
    }
    else if ((name)=="TARGET")
    {
        (*calendar) = TARGET();
        adjustCalendarHolidays("TARGET", calendar);
        return calendar;
    }
    else if ((name)=="Turkey")
    {
        (*calendar) = Turkey();
        adjustCalendarHolidays("Turkey", calendar);
        return calendar;
    }
    else if ((name)=="Ukraine:USE")
    {
        (*calendar) = Ukraine(Ukraine::USE);
        adjustCalendarHolidays("Ukraine:USE", calendar);
        return calendar;
    }
    else if ((name)=="UnitedKingdom:Exchange")
    {
        (*calendar) = UnitedKingdom(UnitedKingdom::Exchange);
        adjustCalendarHolidays("UnitedKingdom:Exchange", calendar);
        return calendar;
    }
    else if ((name)=="UnitedKingdom:Metals")
    {
        (*calendar) = UnitedKingdom(UnitedKingdom::Metals);
        adjustCalendarHolidays("UnitedKingdom:Metals", calendar);
        return calendar;
    }
    else if ((name)=="UnitedKingdom:Settlement")
    {
        (*calendar) = UnitedKingdom(UnitedKingdom::Settlement);
        adjustCalendarHolidays("UnitedKingdom:Settlement", calendar);
        return calendar;
    }
    else if ((name)=="UnitedStates:GovernmentBond")
    {
        (*calendar) = UnitedStates(UnitedStates::GovernmentBond);
        adjustCalendarHolidays("UnitedStates:GovernmentBond", calendar);
        return calendar;
    }
    else if ((name)=="UnitedStates:NERC")
    {
        (*calendar) = UnitedStates(UnitedStates::NERC);
        adjustCalendarHolidays("UnitedStates:NERC", calendar);
        return calendar;
    }
    else if ((name)=="UnitedStates:NYSE")
    {
        (*calendar) = UnitedStates(UnitedStates::NYSE);
        adjustCalendarHolidays("UnitedStates:NYSE", calendar);
        return calendar;
    }
    else if ((name)=="UnitedStates:Settlement")
    {
        (*calendar) = UnitedStates(UnitedStates::Settlement);
        adjustCalendarHolidays("UnitedStates:Settlement", calendar);
        return calendar;
    }
    return NULL;
}

//------------------------------------------------------------------------------
QuantLib::BusinessDayConvention getBusinessConvention(QString convention)
{
    if (convention=="Following")
        return QuantLib::Following;
    else if (convention=="ModifiedFollowing")
        return QuantLib::ModifiedFollowing;
    else if (convention=="ModifiedPreceding")
        return QuantLib::ModifiedPreceding;
    else if (convention=="Preceding")
        return QuantLib::Preceding;
    else if (convention=="Unadjusted")
        return QuantLib::Unadjusted;

    return QuantLib::Unadjusted;
}

//------------------------------------------------------------------------------

QuantLib::DayCounter *getDayCounter(QString calConvention)
{
    using namespace QuantLib;

    if (calConvention=="Actual360")
        return new Actual360();
    else if (calConvention=="Actual365Fixed")
        return new Actual365Fixed();
    else if (calConvention=="ActualActual")
        return new ActualActual();
    else if (calConvention=="Business252")
        return new Business252();
    else if (calConvention=="Thirty360")
        return new Thirty360();
    return NULL;
}

//------------------------------------------------------------------------------
QuantLib::TimeUnit getPeriodo(QString periodo)
{
    using namespace QuantLib;

    if(periodo=="Dia")
        return Days;
    if(periodo=="Semana")
        return Weeks;
    if(periodo=="Mes")
        return Months;
    if(periodo=="Anio")
        return Years;

    return Years; //por defecto
}

//------------------------------------------------------------------------------
boost::shared_ptr<QuantLib::IborIndex> getIndex(QString indexName, QuantLib::Handle<QuantLib::YieldTermStructure> curva)
{
    using namespace QuantLib;
    extern DBHandler *DB;
    QString standardIndex, periodo;
    int plazo=0;
    DB->getStringFieldFromTableforID("indices", "ID", indexName, "INDICE_ESTANDAR", &standardIndex);
    DB->getStringFieldFromTableforID("indices", "ID", indexName, "PERIODO", &periodo);
    DB->getIntFieldFromTableforID("indices", "ID", indexName, "PLAZO", &plazo);

    if ((standardIndex)=="AUDLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new AUDLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="CADLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new CADLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="CDOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new Cdor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="CHFLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new CHFLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="DKKLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new DKKLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="EURIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new Euribor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="EURIBOR365")
    {
        boost::shared_ptr<IborIndex> retIndex(new Euribor365(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="EURLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new EURLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="GBPLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new GBPLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="JIBAR")
    {
        boost::shared_ptr<IborIndex> retIndex(new Jibar(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="JPYLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new JPYLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="NZDLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new NZDLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="SEKLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new SEKLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="TIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new Tibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="TRLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new TRLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="USDLIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new USDLibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }
    if ((standardIndex)=="ZIBOR")
    {
        boost::shared_ptr<IborIndex> retIndex(new Zibor(Period(plazo, getPeriodo(periodo)), curva));
        DB->addIndexFixings(indexName, retIndex);
        return retIndex;
    }

    //por defecto se devuelve euribor
    boost::shared_ptr<IborIndex> retIndex(new Euribor(Period(plazo, getPeriodo(periodo)), curva));
    return retIndex;

}
//------------------------------------------------------------------------------
boost::shared_ptr<QuantLib::RateHelper> getRateFromDepo(double quote, int plazo, QString periodo, unsigned int fixingDays, QuantLib::Calendar *calendar, QuantLib::BusinessDayConvention fixedBusinessConv, QuantLib::DayCounter *fixedLegDayCounter)
{
    using namespace QuantLib;

    boost::shared_ptr<Quote> depoQuote(new SimpleQuote(quote));
    TimeUnit period=getPeriodo(periodo);

    boost::shared_ptr<RateHelper> depoRate(new DepositRateHelper(Handle<Quote>(depoQuote), plazo*period, fixingDays, *calendar, fixedBusinessConv, true, *fixedLegDayCounter));

    return depoRate;
}

//------------------------------------------------------------------------------
boost::shared_ptr<QuantLib::RateHelper> getRateFromFra(double quote, int mStart, int mEnd, unsigned int fixingDays, QuantLib::Calendar *calendar, QuantLib::BusinessDayConvention fixedBusinessConv, QuantLib::DayCounter *fixedLegDayCounter)
{
    using namespace QuantLib;

    boost::shared_ptr<Quote> fraQuote(new SimpleQuote(quote));
    boost::shared_ptr<RateHelper> fraRate(new FraRateHelper(Handle<Quote>(fraQuote), mStart, mEnd, fixingDays, *calendar, fixedBusinessConv, true, *fixedLegDayCounter));
    return fraRate;
}

//------------------------------------------------------------------------------
boost::shared_ptr<QuantLib::RateHelper> getRateFromFuturo(double quote, QuantLib::Date immDate, int months, QuantLib::Calendar *calendar, QuantLib::BusinessDayConvention fixedBusinessConv, QuantLib::DayCounter *fixedLegDayCounter)
{
    using namespace QuantLib;

    boost::shared_ptr<Quote> futuroQuote(new SimpleQuote(quote));
    boost::shared_ptr<RateHelper> futRate(new FuturesRateHelper( Handle<Quote>(futuroQuote), immDate, months, *calendar, fixedBusinessConv, true, *fixedLegDayCounter));
    return futRate;
}

//------------------------------------------------------------------------------
boost::shared_ptr<QuantLib::RateHelper> getRateFromSwap(double quote, int plazo, QString periodo, QuantLib::Calendar *calendar, QuantLib::Frequency fixedLegFrequency, QuantLib::BusinessDayConvention fixedBusinessConv, QuantLib::DayCounter *fixedLegDayCounter, QString indexName)
{
    using namespace QuantLib;

    boost::shared_ptr<Quote> swapQuote(new SimpleQuote(quote));
    boost::shared_ptr<QuantLib::IborIndex> floatingLegIndex=getIndex(indexName, Handle<YieldTermStructure>());
    TimeUnit period=getPeriodo(periodo);
    boost::shared_ptr<RateHelper> swapRate(new SwapRateHelper( Handle<Quote>(swapQuote), plazo*period, *calendar, fixedLegFrequency, fixedBusinessConv, *fixedLegDayCounter, floatingLegIndex));
    return swapRate;
}
//------------------------------------------------------------------------------
boost::shared_ptr<QuantLib::YieldTermStructure> getCurve(QString curveName, QuantLib::Calendar *calendar,
                                               QuantLib::DayCounter *dayCounter,
                                               QuantLib::BusinessDayConvention businessConv,
                                               QuantLib::Frequency frequency, QString indexName)
{
    using namespace QuantLib;


    extern DBHandler *DB;
    QTableWidget *tabla= new QTableWidget;
    QString idProducto;
    QString tipoProducto;
    int plazo=0;
    QString periodo;
    double quote=0.0;
    int mStart=0, mEnd=0, numRows=0;
    QString divisa, immDate;
    unsigned int fixingDays=0;

    std::vector<boost::shared_ptr<RateHelper> > instruments;

    DB->getDataFromCurve(tabla,curveName);
    //PRODUCT_ID;TIPO;DIVISA;START DATE;PLAZO;PERIODO;DISCOUNT FACTOR;MARKdET QUOTE");
    numRows=tabla->rowCount();

    for (int i=0; i<numRows; i++)
    {
        idProducto=tabla->item(i,0)->text();
        tipoProducto=tabla->item(i,1)->text();
        divisa=tabla->item(i,2)->text();
        plazo=tabla->item(i,4)->text().toInt();
        periodo=tabla->item(i,5)->text();
        quote=tabla->item(i,6)->text().toDouble();
        fixingDays=DB->getCurrencySpotSchedule(divisa);

        if (tipoProducto=="Deposito")
        {
            boost::shared_ptr<RateHelper> tmp=getRateFromDepo(quote, plazo, periodo, fixingDays, calendar, businessConv, dayCounter);
            instruments.push_back(tmp);
        }
        else if (tipoProducto=="Fra")
        {
            DB->getIntFieldFromTableforID("fras", "ID", idProducto, "MONTHSTOSTART", &mStart);
            DB->getIntFieldFromTableforID("fras", "ID", idProducto, "MONTHSTOEND", &mEnd);
            boost::shared_ptr<RateHelper> tmp=getRateFromFra(quote, mStart, mEnd, fixingDays, calendar, businessConv, dayCounter);
            instruments.push_back(tmp);
        }
        else if (tipoProducto=="Futuro")
        {
            //boost::shared_ptr<RateHelper> fut2(new FuturesRateHelper( Handle<Quote>(fut2Price), imm, futMonths, calendar, ModifiedFollowing, true, depositDayCounter));
            DB->getStringFieldFromTableforID("futuros", "ID", idProducto, "IMM_DATE", &immDate);
            QStringList lista=immDate.split("-");
            int months=0;
            DB->getIntFieldFromTableforID("futuros", "ID", idProducto, "MONTHS", &months);
            int year=lista.at(0).toInt();
            int month=lista.at(1).toInt();
            int day=lista.at(2).toInt();
            Date imm=Date(day, (Month)month, year);
            boost::shared_ptr<RateHelper> tmp=getRateFromFuturo(quote, imm, months, calendar, businessConv, dayCounter);
            instruments.push_back(tmp);
        }
        else if (tipoProducto=="Swap")
        {
            boost::shared_ptr<RateHelper> tmp=getRateFromSwap(quote, plazo, periodo, calendar, frequency, businessConv, dayCounter, indexName);
            instruments.push_back(tmp);
        }
    }
    double tolerance=0.;
    DB->getDoubleFieldFromTableforID("curvas", "ID", curveName, "TOLERANCIA", &tolerance);

    QString convCal;
    DB->getStringFieldFromTableforID("curvas", "ID", curveName, "CONVENCION_CALENDARIO", &convCal);
    DayCounter *dayCount=getDayCounter(convCal);


    Date today = calendar->adjust(Date::todaysDate());
    Settings::instance().evaluationDate() = today;
    boost::shared_ptr<YieldTermStructure> curve( new PiecewiseYieldCurve<Discount,LogLinear>( today, instruments, *dayCount, tolerance));
    delete dayCount;
    delete tabla;
    return curve;
}

//------------------------------------------------------------------------------
QuantLib::VanillaSwap *getIRS(double nominal, QDate settlementDate, QDate maturityDate, QString currency, QString rol, double spread,
                    bool isForward, QDate fwdStart, QString fixedFrequency, QString fixedBusinessConv, QString fixedCalConv,
                    double fixedRate, QString discountCurveName, QString floatingFrequency, QString floatingBusinessConv,
                    QString floatingCalConv, QString indexName, QString forwardCurveName)
{
    using namespace QuantLib;
    extern DBHandler *DB;
    QString *name=DB->getCalendarFromCurrency(currency);
    Calendar *calendar = getCalendar(*name);
    delete name;

    unsigned int fixingDays=DB->getCurrencySpotSchedule(currency);

    Date settlement(settlementDate.day(), (Month)settlementDate.month(), settlementDate.year());
    settlement = calendar->advance(settlement, fixingDays, Days);
    settlement = calendar->adjust(settlement);
    Date maturity(maturityDate.day(), (Month)maturityDate.month(), maturityDate.year());
    Date today=Date::todaysDate();
    Settings::instance().evaluationDate() = today;
//    if (settlement<today)
//    {
//        QMessageBox::critical(0,"Error en valoración","La fecha de contratacion no puede ser anterior a hoy");
//        return NULL;
//    }



    Frequency fixFrequency = getFrequency(fixedFrequency);
    Frequency floatFrequency = getFrequency(floatingFrequency);
    BusinessDayConvention fixBusinessConv = getBusinessConvention(fixedBusinessConv);
    BusinessDayConvention floatBusinessConv = getBusinessConvention(floatingBusinessConv);
    DayCounter *fixedLegDayCounter=getDayCounter(fixedCalConv);
    DayCounter *floatingDayCounter=getDayCounter(floatingCalConv);

    if (isForward)
    {
        Date forwardStart(fwdStart.day(), (Month)fwdStart.month(), fwdStart.year());
        long int diff=forwardStart.serialNumber()-settlement.serialNumber();
        settlement+=diff;
        maturity+=diff;
    }
    Schedule fixedSchedule(settlement, maturity, Period(fixFrequency), *calendar, fixBusinessConv, fixBusinessConv, DateGeneration::Forward, false);
    Schedule floatSchedule(settlement, maturity, Period(floatFrequency), *calendar, floatBusinessConv, floatBusinessConv, DateGeneration::Forward, false);

    boost::shared_ptr<YieldTermStructure> forwardCurve=getCurve(forwardCurveName, calendar, floatingDayCounter, floatBusinessConv, floatFrequency, indexName);
    boost::shared_ptr<YieldTermStructure> discountCurve=getCurve(discountCurveName, calendar, fixedLegDayCounter, fixBusinessConv, fixFrequency, indexName);
    RelinkableHandle<YieldTermStructure> discountingTermStructure;
    RelinkableHandle<YieldTermStructure> forecastingTermStructure;
    forecastingTermStructure.linkTo(forwardCurve);
    discountingTermStructure.linkTo(discountCurve);

    VanillaSwap::Type swapType=getSwapType(rol);

    boost::shared_ptr<QuantLib::IborIndex> floatingLegIndex=getIndex(indexName, Handle<YieldTermStructure>(forwardCurve));
    //Pasamos a tanto por 1
    if (fixedRate)
        fixedRate=fixedRate/100;
    if (spread)
        spread=spread/100;
    VanillaSwap *swap = new VanillaSwap(swapType, nominal, fixedSchedule, fixedRate, *fixedLegDayCounter, floatSchedule, floatingLegIndex, spread, *floatingDayCounter);
    boost::shared_ptr<PricingEngine> swapEngine(new DiscountingSwapEngine(discountingTermStructure));
    swap->setPricingEngine(swapEngine);
    delete fixedLegDayCounter;
    delete floatingDayCounter;
    //No es usado por la curva
    delete calendar;
    return swap;
}

//------------------------------------------------------------------------------
QuantLib::VanillaOption *getEuropeanOption( double nominal, QDate settlementDate, QDate maturityDate,
                                          QString currency, QString tipoOpcion, double spot,
                                          double optStrike, double tipoLibre, double volatilidad, QString convencionCalendario)
{
    using namespace QuantLib;
    extern DBHandler *DB;

    QString *name=DB->getCalendarFromCurrency(currency);
    Calendar *calendar = getCalendar(*name);
    Option::Type type;

    Date today=Date::todaysDate();
    Settings::instance().evaluationDate() = today;
    Date maturity(maturityDate.day(), (Month)maturityDate.month(), maturityDate.year());

    DayCounter *dayCounter=getDayCounter(convencionCalendario);


    if (tipoOpcion=="Put")
        type=Option::Type(Option::Put);
    else
        type=Option::Type(Option::Call);

    Real underlying = spot;
    Real strike = optStrike;
    Spread dividendYield = 0.00;
    Rate riskFreeRate = tipoLibre;
    Volatility volatility = volatilidad/100;

    boost::shared_ptr<Exercise> europeanExercise(new EuropeanExercise(maturity));

    Handle<Quote> underlyingH(boost::shared_ptr<Quote>(new SimpleQuote(underlying)));

    Handle<YieldTermStructure> flatTermStructure(boost::shared_ptr<YieldTermStructure>(new FlatForward(today,
                                                                                                       riskFreeRate,
                                                                                                       *dayCounter)));
    //Dividendo 0 al ser una opción de tipos.
    Handle<YieldTermStructure> flatDividendTermStructure(boost::shared_ptr<YieldTermStructure>(new FlatForward(today,
                                                                                                               dividendYield,
                                                                                                               *dayCounter)));

    Handle<BlackVolTermStructure> flatVolTermStructure(boost::shared_ptr<BlackVolTermStructure>(new BlackConstantVol(today,
                                                                                                                     *calendar,
                                                                                                                     volatility,
                                                                                                                     *dayCounter)));
    boost::shared_ptr<StrikedTypePayoff> payoff(new PlainVanillaPayoff(type,strike));

    // Modelo Black Scholes
    boost::shared_ptr<BlackScholesMertonProcess> bsmProcess(new BlackScholesMertonProcess(  underlyingH,
                                                                                            flatDividendTermStructure,
                                                                                            flatTermStructure,
                                                                                            flatVolTermStructure));

    VanillaOption *europeanOption = new VanillaOption(payoff, europeanExercise);

    europeanOption->setPricingEngine(boost::shared_ptr<PricingEngine>(new AnalyticEuropeanEngine(bsmProcess)));
    delete name;
    delete calendar;
    delete dayCounter;
    return europeanOption;

}

//------------------------------------------------------------------------------
bool getMultipleValoration(QStringList deals, double *NPV, double *delta, double *gamma, double *rho, double *theta, double *vega, QStringList *errores)
{
    extern DBHandler *DB;
    extern QString defaultCurrency;
    QString producto;
    int numDeals=deals.count();
    bool bret=false, isFwd;
    double NPVAgregado=0., nominal=0., spot=0., strike=0., tipoLibre=0., volatilidad=0., spread=0., tipoFijo=0., totalDelta=0., totalGamma=0., totalRho=0., totalTheta=0., totalVega=0.;
    QDate settlement, maturity, fwdStart;
    QString divisa, tipoOpcion, tipoSubyacente, subyacente, convencionCalendario, convencionCalendarioFija, txt, rol, frecuenciaFija, convencionFestivosFija;
    QString curvaDescuento, frecuenciaFlotante, convencionFestivosFlotante, convencionCalendarioFlotante, indice, curvaForward;
    DB->getDefaultCurrency(&defaultCurrency);
    if (!errores)
        errores=new QStringList();

    for (int i=0; i<numDeals; i++)
    {
        try
        {
            DB->getStringFieldFromTableforID("deals", "ID", deals.at(i), "PRODUCT", &producto);
            if (producto=="European Option")
            {

                bret=DB->getEuropeanOption( deals.at(i).toInt(), &nominal, &settlement, &maturity, &divisa, &tipoOpcion, &tipoSubyacente,
                                        &subyacente, &spot, &strike, &tipoLibre, &volatilidad, &convencionCalendario);
                if (!bret)
                {
                    QString txt=QString("Deal %1, Error: %2").arg(deals.at(i)).arg("No es posible generar el producto");
                    errores->append(txt);
                    continue;
                }
                QuantLib::VanillaOption *EO=getEuropeanOption( nominal, settlement, maturity, divisa, tipoOpcion, spot,
                                                               strike, tipoLibre, volatilidad, convencionCalendario);
                if (!EO)
                {
                    QString txt=QString("Deal %1, Error: %2").arg(deals.at(i)).arg("No es posible generar el producto");
                    errores->append(txt);
                    continue;
                }
                double cambio=DB->getTipoCambio(defaultCurrency, divisa);
                if (!cambio)
                {
                    QString txt=QString("Deal %1, Error: %2").arg(deals.at(i)).arg("No se encuentra tipo de cambio para la divisa estandar");
                    errores->append(txt);
                }
                NPVAgregado+=EO->NPV()*nominal*cambio;
                totalDelta+=EO->delta();
                totalGamma+=EO->gamma();
                totalVega+=EO->vega()/100;
                totalTheta+=EO->thetaPerDay();
                totalRho+=EO->rho()/100;
                delete EO;
            }
            else if (producto=="IRS")
            {
                bret=DB->getIRS(deals.at(i).toInt(), &nominal, &settlement, &maturity, &divisa, &rol, &spread, &isFwd, &fwdStart,
                            &frecuenciaFija, &convencionFestivosFija, &convencionCalendarioFija, &tipoFijo, &curvaDescuento,
                            &frecuenciaFlotante,&convencionFestivosFlotante, &convencionCalendarioFlotante, &indice, &curvaForward);
                if (!bret)
                {
                    QString txt=QString("Deal %1, Error: %2").arg(deals.at(i)).arg("No es posible generar el producto");
                    errores->append(txt);
                    continue;
                }

                QuantLib::VanillaSwap *irs=getIRS(nominal, settlement, maturity, divisa, rol, spread, isFwd, fwdStart,
                            frecuenciaFija, convencionFestivosFija, convencionCalendarioFija, tipoFijo, curvaDescuento, frecuenciaFlotante,
                            convencionFestivosFlotante, convencionCalendarioFlotante, indice, curvaForward);
                if (!irs)
                {
                    txt=QString("Deal %1, Error: %2").arg(deals.at(i)).arg("No es posible generar el producto");
                    errores->append(txt);
                    continue;
                }
                double cambio=DB->getTipoCambio(defaultCurrency, divisa);
                //Aqui no multiplicamos por nominal ya que irs ya lo tiene en cuenta.
                if (!cambio)
                {
                    QString txt=QString("Deal %1, Error: %2").arg(deals.at(i)).arg("No se encuentra tipo de cambio para la divisa estandar");
                    errores->append(txt);
                }

                NPVAgregado+=irs->NPV()*cambio;
                delete irs;
            }
        }
        catch (std::exception& e)
        {
            txt=QString("Deal %1, Error: %2").arg(deals.at(i)).arg(e.what());
            errores->append(txt);
            continue;
        }
        catch (...)
        {
            continue;
        }


    }
    *NPV=NPVAgregado;
    *delta=totalDelta;
    *gamma=totalGamma;
    *vega=totalVega;
    *theta=totalTheta;
    *rho=totalRho;

    return true;
}

